﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.DAL.Services;
using Sage.PL.WebAPI.Interface.Contractors.Interfaces;
using Sage.PL.WebAPI.Interface.Contractors.ViewModels;

namespace Sage.PL.WebAPI.Demo.DAL.Repositories
{
	public class ContractorsRepository
	{
		private readonly IContractorsController contractorsController;
		private readonly HttpResponseService httpResponseService;

		public ContractorsRepository(IContractorsController contractorsController, HttpResponseService httpResponseService)
		{
			this.contractorsController = contractorsController;
			this.httpResponseService = httpResponseService;
		}

		public ResponseResult<Contractor> AddNew(Contractor contractor)
		{
			ResponseResult<Contractor> result;
			var message = contractorsController.AddNew(contractor) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Contractor>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Contractor> Update(Contractor contractor)
		{
			ResponseResult<Contractor> result;
			var message = contractorsController.Update(contractor) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Contractor>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Contractor> Get(int id)
		{
			ResponseResult<Contractor> result;
			var message = contractorsController.Get(id) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Contractor>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Contractor> Get(string code)
		{
			ResponseResult<Contractor> result;
			var message = contractorsController.Get(code) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Contractor>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<ContractorListElement>> Get()
		{
			ResponseResult<List<ContractorListElement>> result;
			var message = contractorsController.Get() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<ContractorListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
	}
}
