﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.DAL.Services;
using Sage.PL.WebAPI.Interface.FKF.Documents.Interfaces;
using Sage.PL.WebAPI.Interface.FKF.Documents.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Documents.ViewModels.Issue.Custom;

namespace Sage.PL.WebAPI.Demo.DAL.Repositories
{
	public class FKDocumentsRepository
	{
		private readonly IFKDocumentsController documentsController;
		private readonly IFKDocumentsIssueController documentsIssueController;
		private readonly HttpResponseService httpResponseService;

		public FKDocumentsRepository(IFKDocumentsController documentsController, IFKDocumentsIssueController documentsIssueController, HttpResponseService httpResponseService)
		{
			this.documentsController = documentsController;
			this.documentsIssueController = documentsIssueController;
			this.httpResponseService = httpResponseService;
		}

		public ResponseResult<List<DocumentListElement>> Get()
		{
			ResponseResult<List<DocumentListElement>> result;
			var message = documentsController.Get() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<DocumentListElement>> GetFromBuffer(int yearId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			ResponseResult<List<DocumentListElement>> result;
			var message = documentsController.GetFromBuffer(yearId, dateFrom, dateTo) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<DocumentType>> GetDocumentTypes(int yearId)
		{
			ResponseResult<List<DocumentType>> result;
			var message = documentsController.GetDocumentTypes(yearId) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentType>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}

		public ResponseResult<Document> AddNew(DocumentIssue order)
		{
			ResponseResult<Document> result;
			var message = documentsIssueController.AddNew(order) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Document>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
	}
}
