﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.DAL.Services;
using Sage.PL.WebAPI.Interface.FKF.Contractors.Interfaces;
using Sage.PL.WebAPI.Interface.FKF.Contractors.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Employees.Interfaces;
using Sage.PL.WebAPI.Interface.FKF.Employees.ViewModels;

namespace Sage.PL.WebAPI.Demo.DAL.Repositories
{
	public class FKEmployeesRepository
	{
		private readonly IFKEmployeesController employeesController;
		private readonly HttpResponseService httpResponseService;

		public FKEmployeesRepository(IFKEmployeesController employeesController, HttpResponseService httpResponseService)
		{
			this.employeesController = employeesController;
			this.httpResponseService = httpResponseService;
		}

		public ResponseResult<Employee> AddNew(Employee employee)
		{
			ResponseResult<Employee> result;
			var message = employeesController.AddNew(employee) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Employee>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Employee> Update(Employee employee)
		{
			ResponseResult<Employee> result;
			var message = employeesController.Update(employee) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Employee>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Employee> GetById(int id)
		{
			ResponseResult<Employee> result;
			var message = employeesController.GetById(id) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Employee>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Employee> GetByPosition(int position)
		{
			ResponseResult<Employee> result;
			var message = employeesController.GetByPosition(position) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Employee>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Employee> GetByCode(string code)
		{
			ResponseResult<Employee> result;
			var message = employeesController.GetByCode(code) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Employee>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<EmployeeListElement>> Get()
		{
			ResponseResult<List<EmployeeListElement>> result;
			var message = employeesController.Get() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<EmployeeListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
	}
}
