﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.DAL.Services;
using Sage.PL.WebAPI.Interface.Common.ViewModels;
using Sage.PL.WebAPI.Interface.Orders.Interfaces;
using Sage.PL.WebAPI.Interface.Orders.ViewModels;

namespace Sage.PL.WebAPI.Demo.DAL.Repositories
{
	public class ForeignOrdersRepository
	{
		private readonly IOrdersController ordersController;
		private readonly IOrdersIssueController ordersIssueController;
		private readonly HttpResponseService httpResponseService;

		public ForeignOrdersRepository(IOrdersController ordersController, IOrdersIssueController ordersIssueController, HttpResponseService httpResponseService)
		{
			this.ordersController = ordersController;
			this.ordersIssueController = ordersIssueController;
			this.httpResponseService = httpResponseService;
		}

		public ResponseResult<Order> AddNew(OrderIssue order)
		{
			ResponseResult<Order> result;
			var message = ordersIssueController.AddNew(order) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Order>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}

		public ResponseResult<List<OrderListElement>> Get()
		{
			ResponseResult<List<OrderListElement>> result;
			var message = ordersController.Get() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<OrderListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<OrderListElement>> GetList(DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			ResponseResult<List<OrderListElement>> result;
			var message = ordersController.GetList(dateFrom, dateTo) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<OrderListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}

		public ResponseResult<List<DocumentType>> GetZMODocumentTypes()
		{
			ResponseResult<List<DocumentType>> result;
			var message = ordersController.GetZMODocumentTypes() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentType>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<DocumentType>> GetZWODocumentTypes()
		{
			ResponseResult<List<DocumentType>> result;
			var message = ordersController.GetZWODocumentTypes() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentType>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<DocumentSeries>> GetDocumentSeries(int documentTypeId)
		{
			ResponseResult<List<DocumentSeries>> result;
			var message = ordersController.GetDocumentSeries(documentTypeId) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentSeries>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<PDF> GetPDF(int orderId)
		{
			ResponseResult<PDF> result;
			var message = ordersController.GetPDF(orderId) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<PDF>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
	}
}
