﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.DAL.Services;
using Sage.PL.WebAPI.Interface.Products.Interfaces;
using Sage.PL.WebAPI.Interface.Products.ViewModels;

namespace Sage.PL.WebAPI.Demo.DAL.Repositories
{
	public class ProductsRepository
	{
		private readonly IProductsController productsController;
		private readonly HttpResponseService httpResponseService;

		public ProductsRepository(IProductsController productsController, HttpResponseService httpResponseService)
		{
			this.productsController = productsController;
			this.httpResponseService = httpResponseService;
		}

		public ResponseResult<Product> AddNew(Product Product)
		{
			ResponseResult<Product> result;
			var message = productsController.AddNew(Product) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Product>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Product> Update(Product Product)
		{
			ResponseResult<Product> result;
			var message = productsController.Update(Product) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Product>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Product> Get(int id)
		{
			ResponseResult<Product> result;
			var message = productsController.Get(id) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Product>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<Product> Get(string code)
		{
			ResponseResult<Product> result;
			var message = productsController.Get(code) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<Product>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<ProductListElement>> Get()
		{
			ResponseResult<List<ProductListElement>> result;
			var message = productsController.Get() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<ProductListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
	}
}
