﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.DAL.Services;
using Sage.PL.WebAPI.Interface.Common.ViewModels;
using Sage.PL.WebAPI.Interface.Orders.Interfaces;
using Sage.PL.WebAPI.Interface.Orders.ViewModels;
using Sage.PL.WebAPI.Interface.Purchases.Interfaces;
using Sage.PL.WebAPI.Interface.Purchases.ViewModels;

namespace Sage.PL.WebAPI.Demo.DAL.Repositories
{
	public class PurchaseDocumentRepository
	{
		private readonly IPurchasesController purchasesController;
		private readonly IPurchasesIssueController purchasesIssueController;
		private readonly HttpResponseService httpResponseService;

		public PurchaseDocumentRepository(IPurchasesController purchasesController, IPurchasesIssueController purchasesIssueController, HttpResponseService httpResponseService)
		{
			this.purchasesController = purchasesController;
			this.purchasesIssueController = purchasesIssueController;
			this.httpResponseService = httpResponseService;
		}

		public ResponseResult<PurchaseDocument> AddNew(PurchaseDocumentIssue order)
		{
			ResponseResult<PurchaseDocument> result;
			var message = purchasesIssueController.AddNew(order, true) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<PurchaseDocument>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}

		public ResponseResult<List<PurchaseDocumentListElement>> Get()
		{
			ResponseResult<List<PurchaseDocumentListElement>> result;
			var message = purchasesController.Get() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<PurchaseDocumentListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<PurchaseDocumentListElement>> GetList(DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			ResponseResult<List<PurchaseDocumentListElement>> result;
			var message = purchasesController.GetList(dateFrom, dateTo) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<PurchaseDocumentListElement>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}

		public ResponseResult<List<DocumentType>> GetFVZDocumentTypes()
		{
			ResponseResult<List<DocumentType>> result;
			var message = purchasesController.GetFVZDocumentTypes() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentType>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<DocumentType>> GetFWZDocumentTypes()
		{
			ResponseResult<List<DocumentType>> result;
			var message = purchasesController.GetFWZDocumentTypes() as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentType>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<List<DocumentSeries>> GetDocumentSeries(int documentTypeId)
		{
			ResponseResult<List<DocumentSeries>> result;
			var message = purchasesController.GetDocumentSeries(documentTypeId) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<List<DocumentSeries>>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
		public ResponseResult<PDF> GetPDF(int orderId)
		{
			ResponseResult<PDF> result;
			var message = purchasesController.GetPDF(orderId) as ResponseMessageResult;
			if (message != null)
				result = httpResponseService.ParseResponse<PDF>(message.Response);
			else
				throw new ArgumentNullException(nameof(message));

			return result;
		}
	}
}
