﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Interchange.Extensions;
using Sage.PL.WebAPI.Interface.Contractors.Interfaces;
using Sage.PL.WebAPI.Interface.Contractors.ViewModels;
using Unity.Attributes;

namespace Sage.PL.WebAPI.Demo.Interchange.Handlers
{
	public class ContractorsHandler : IContractorsController
	{
		private readonly HttpClient httpClient;
		private readonly string sessionGuid;

		public ContractorsHandler(HttpClient httpClient, [Dependency("SessionGuid")] string sessionGuid)
		{
			this.httpClient = httpClient;
			this.sessionGuid = sessionGuid;
		}

		public IHttpActionResult AddNew(Contractor contractor, bool syncFk = false)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Contractors/Create").AppendQuery(nameof(syncFk), syncFk.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Post;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
			request.Content = new ObjectContent<Contractor>(contractor, new JsonMediaTypeFormatter(), new MediaTypeHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Update(Contractor contractor, bool syncFk = false)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Contractors/Update").AppendQuery(nameof(syncFk), syncFk.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Put;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
			request.Content = new ObjectContent<Contractor>(contractor, new JsonMediaTypeFormatter(), new MediaTypeHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get(int id)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Contractors").AppendQuery(nameof(id), id.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get(string code)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Contractors").AppendQuery(nameof(code), code);
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Contractors");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}

		public IHttpActionResult GetCatalogs()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetKinds()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetMarkers()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetByNIP(string nip)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetByPesel(string pesel)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetByPosition(int position)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetByNIPAndPosition(string nip, int position)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult Filter([FromBody] ContractorFilterCriteria criteria)
		{
			throw new NotImplementedException();
		}
	}
}
