﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Interchange.Extensions;
using Sage.PL.WebAPI.Interface.FKF.Common.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Contractors.Interfaces;
using Sage.PL.WebAPI.Interface.FKF.Contractors.ViewModels;
using Unity.Attributes;

namespace Sage.PL.WebAPI.Demo.Interchange.Handlers
{
	public class FKContractorsHandler : IFKContractorsController
	{
		private readonly HttpClient httpClient;
		private readonly string sessionGuid;

		public FKContractorsHandler(HttpClient httpClient, [Dependency("SessionGuid")]string sessionGuid)
		{
			this.httpClient = httpClient;
			this.sessionGuid = sessionGuid;
		}

		public IHttpActionResult Get()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKContractors");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetById(int id)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKContractors").AppendQuery(nameof(id), id.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetByPosition(int position)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKContractors").AppendQuery(nameof(position), position.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetByCode(string code)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKContractors").AppendQuery(nameof(code), code);
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult AddNew([FromBody] Contractor contractor)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKContractors/Create");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Post;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
			request.Content = new ObjectContent<Contractor>(contractor, new JsonMediaTypeFormatter(), new MediaTypeHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Update([FromBody] Contractor contractor)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKContractors/Update");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Put;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
			request.Content = new ObjectContent<Contractor>(contractor, new JsonMediaTypeFormatter(), new MediaTypeHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}

		public IHttpActionResult GetCatalogs()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetKinds()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetMarkers()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetByNIP(string nip)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetByPesel(string pesel)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult FilterOptions()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult Filter([FromBody] List<FilterCriteria> criteria)
		{
			throw new NotImplementedException();
		}
	}
}
