﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Interchange.Extensions;
using Sage.PL.WebAPI.Interface.FKF.Documents.Interfaces;
using Sage.PL.WebAPI.Interface.Orders.Interfaces;
using Unity.Attributes;

namespace Sage.PL.WebAPI.Demo.Interchange.Handlers
{
	public class FKDocumentsHandler : IFKDocumentsController
	{
		private readonly HttpClient httpClient;
		private readonly string sessionGuid;

		public FKDocumentsHandler(HttpClient httpClient, [Dependency("SessionGuid")]string sessionGuid)
		{
			this.httpClient = httpClient;
			this.sessionGuid = sessionGuid;
		}

		public IHttpActionResult Get()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKDocuments");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get(int id, int yearId)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKDocuments").AppendQuery(nameof(id), id.ToString()).AppendQuery(nameof(yearId), yearId.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetFromBuffer(int yearId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKDocuments/Buffer").AppendQuery(nameof(yearId), yearId.ToString());
			if (dateFrom.HasValue)
				uri = uri.AppendQuery(nameof(dateFrom), dateFrom.Value.ToString("yyyy-MM-dd"));
			if (dateTo.HasValue)
				uri = uri.AppendQuery(nameof(dateTo), dateTo.Value.ToString("yyyy-MM-dd"));

			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetDocumentTypes(int yearId)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/FKDocuments/DocumentTypes").AppendQuery(nameof(yearId), yearId.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}

		public IHttpActionResult GetFromAccountingBooks(int yearId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFromSchemes(int yearId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFromBuffer(int yearId, int contractorPosition, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFromAccountingBooks(int yearId, int contractorPosition, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFromSchemes(int yearId, int contractorPosition, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetDocumentFeatures()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetDocumentFeatures(int yearId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetDocumentRecordFeatures()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetDocumentRecordFeatures(int yearId)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetDocumentTypes()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetMarkers()
		{
			throw new NotImplementedException();
		}
	}
}
