﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Interchange.Extensions;
using Sage.PL.WebAPI.Interface.Orders.Interfaces;
using Unity.Attributes;

namespace Sage.PL.WebAPI.Demo.Interchange.Handlers
{
	public class ForeignOrdersHandler : IOrdersController
	{
		private readonly HttpClient httpClient;
		private readonly string sessionGuid;

		public ForeignOrdersHandler(HttpClient httpClient, [Dependency("SessionGuid")] string sessionGuid)
		{
			this.httpClient = httpClient;
			this.sessionGuid = sessionGuid;
		}

		public IHttpActionResult Get()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Orders");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetList(DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Orders/Filter");
			if (dateFrom.HasValue)
				uri = uri.AppendQuery(nameof(dateFrom), dateFrom.Value.ToString("yyyy-MM-dd"));
			if (dateTo.HasValue)
				uri = uri.AppendQuery(nameof(dateTo), dateTo.Value.ToString("yyyy-MM-dd"));

			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetZMODocumentTypes()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Orders/ZMODocumentTypes");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetDocumentSeries(int documentTypeId)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Orders/DocumentSeries").AppendQuery(nameof(documentTypeId), documentTypeId.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetPDF(int orderId, bool printNote = false)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Orders/PDF").AppendQuery(nameof(orderId), orderId.ToString()).AppendQuery(nameof(printNote), printNote.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}

		public IHttpActionResult Get(int id)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Orders").AppendQuery(nameof(id), id.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get(string number, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFV(int orderId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFV(string orderNumber, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListByBuyer(int buyerId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListByBuyer(string buyerCode, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListByRecipient(int recipientId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListByRecipient(string recipientCode, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetStatus(int orderId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetStatus(string orderNumber, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetWZ(int orderId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetWZ(string orderNumber, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetZWODocumentTypes()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Orders/ZWODocumentTypes");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetCatalogs()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetKinds()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetMarkers()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetListByDimension(string dimensionCode, string value, string dictionaryValue = null)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetPDF(string orderNumber, bool buffer = false, bool printNote = false)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetInvoicesPDF(int orderId, bool printNote = false)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetInvoicesPDF(string orderNumber, bool buffer = false, bool printNote = false)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetPositionRelations(int positionId)
		{
			throw new NotImplementedException();
		}
	}
}
