﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Interchange.Extensions;
using Sage.PL.WebAPI.Interface.Products.Interfaces;
using Sage.PL.WebAPI.Interface.Products.ViewModels;
using Unity.Attributes;

namespace Sage.PL.WebAPI.Demo.Interchange.Handlers
{
	public class ProductsHandler : IProductsController
	{
		private readonly HttpClient httpClient;
		private readonly string sessionGuid;

		public ProductsHandler(HttpClient httpClient, [Dependency("SessionGuid")] string sessionGuid)
		{
			this.httpClient = httpClient;
			this.sessionGuid = sessionGuid;
		}

		public IHttpActionResult AddNew(Product product)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Products/Create");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Post;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
			request.Content = new ObjectContent<Product>(product, new JsonMediaTypeFormatter(), new MediaTypeHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Update(Product product)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Products/Update");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Put;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
			request.Content = new ObjectContent<Product>(product, new JsonMediaTypeFormatter(), new MediaTypeHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get(int id)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Products").AppendQuery(nameof(id), id.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get(string code)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Products").AppendQuery(nameof(code), code);
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult Get(bool salePrices = false)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Products");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}

		public IHttpActionResult GetCatalogs()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetKinds()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetMarkers()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult Filter([FromBody] ProductFilterCriteria criteria, bool salePrices = false)
		{
			throw new NotImplementedException();
		}
	}
}
