﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Interchange.Extensions;
using Sage.PL.WebAPI.Interface.Orders.Interfaces;
using Sage.PL.WebAPI.Interface.Purchases.Interfaces;
using Unity.Attributes;

namespace Sage.PL.WebAPI.Demo.Interchange.Handlers
{
	public class PurchaseDocumentsHandler : IPurchasesController
	{
		private readonly HttpClient httpClient;
		private readonly string sessionGuid;

		public PurchaseDocumentsHandler(HttpClient httpClient, [Dependency("SessionGuid")] string sessionGuid)
		{
			this.httpClient = httpClient;
			this.sessionGuid = sessionGuid;
		}

		public IHttpActionResult Get()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Purchases");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetList(DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Purchases/Filter");
			if (dateFrom.HasValue)
				uri = uri.AppendQuery(nameof(dateFrom), dateFrom.Value.ToString("yyyy-MM-dd"));
			if (dateTo.HasValue)
				uri = uri.AppendQuery(nameof(dateTo), dateTo.Value.ToString("yyyy-MM-dd"));

			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetFVZDocumentTypes()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Purchases/FVZDocumentTypes");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetFWZDocumentTypes()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Purchases/FWZDocumentTypes");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetDocumentSeries(int documentTypeId)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Purchases/DocumentSeries").AppendQuery(nameof(documentTypeId), documentTypeId.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult GetPDF(int documentId, bool printNote = false)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Purchases/PDF").AppendQuery(nameof(documentId), documentId.ToString()).AppendQuery(nameof(printNote), printNote.ToString());
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}



		public IHttpActionResult Get(int id)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult Get(string number, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetCatalogs()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetKinds()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetMarkers()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetCorrection(int id)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetCorrection(string number, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetCorrectionSequence(int documentId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetCorrectionSequence(string documentNumber, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetDIMDocumentTypes()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFVMDocumentTypes()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetFVRDocumentTypes()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetRUZDocumentTypes()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetWNTDocumentTypes()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetZRZDocumentTypes()
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListByDeliverer(int delivererId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListByDeliverer(string delivererCode, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListByDimension(string dimensionCode, string value, string dictionaryValue = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListBySeller(int sellerId, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetListBySeller(string sellerCode, DateTime? dateFrom = null, DateTime? dateTo = null)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetPDF(string documentNumber, bool buffer = false, bool printNote = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetPZ(int documentId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetPZ(string documentNumber, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetStatus(int documentId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetStatus(string documentNumber, bool buffer = false)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetZMW(int documentId)
		{
			throw new NotImplementedException();
		}
		public IHttpActionResult GetZMW(string documentNumber, bool buffer = false)
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetDMKDocumentTypes()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetFMKDocumentTypes()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetFRKDocumentTypes()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetFZKDocumentTypes()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetFWZKDocumentTypes()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetRKZDocumentTypes()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetZRKDocumentTypes()
		{
			throw new NotImplementedException();
		}

		public IHttpActionResult GetWNKDocumentTypes()
		{
			throw new NotImplementedException();
		}
	}
}
