﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Interchange.Extensions;
using Sage.PL.WebAPI.Interface.Interfaces;
using Unity.Attributes;

namespace Sage.PL.WebAPI.Demo.Interchange.Handlers
{
	public class SessionHandler : ISessionController
	{
		private readonly string applicationGuid;
		private readonly string sessionGuid;
		private readonly HttpClient httpClient;

		public SessionHandler(HttpClient httpClient, [Dependency("AppGuid")]string applicationGuid, [Dependency("SessionGuid")]string sessionGuid)
		{
			this.httpClient = httpClient;
			this.applicationGuid = applicationGuid;
			this.sessionGuid = sessionGuid;
		}

		public IHttpActionResult OpenNewSession(string deviceName)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Sessions/OpenNewSession").AppendQuery(nameof(deviceName), deviceName);
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Application", applicationGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult CloseSession()
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Sessions/CloseSession");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult CloseSession(string _sessionGuid)
		{
			Uri uri = httpClient.BaseAddress.SetPath("/api/Sessions/CloseSession");
			var request = new HttpRequestMessage();
			request.RequestUri = uri;
			request.Method = HttpMethod.Get;
			request.Headers.Authorization = new AuthenticationHeaderValue("Session", _sessionGuid);
			request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

			var response = httpClient.SendAsync(request);
			return new ResponseMessageResult(response.Result);
		}
		public IHttpActionResult SessionInformation()
		{
			throw new NotImplementedException();
		}
	}
}
