﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Sage.PL.WebAPI.Interface.Contractors.ViewModels;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Demo.Extensions;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using DevExpress.XtraEditors;
using Sage.PL.WebAPI.Demo.Forms;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.DAL;
using Sage.PL.WebAPI.Interface.Common.ViewModels;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlContractor : ManagedControl
	{
		public override Guid Identifier => new Guid("644E80B6-35D7-4C70-90AD-B36C607BDDB0");

		private Contractor contractor;
		private readonly ContractorsRepository contractorsRepository;
		private readonly CountriesRepository countriesRepository;
		private readonly RegionsRepository regionsRepository;
		private readonly PaymentFormsRepository paymentFormsRepository;
		private readonly PaymentRegistriesRepository paymentRegistriesRepository;
		private readonly PricesRepository pricesRepository;
		private readonly CurrenciesRepository currenciesRepository;
		private readonly PriceTypeMapper priceTypeMapper;
		private readonly MessagesService messagesService;

		public ControlContractor(Contractor contractor, ContractorsRepository contractorsRepository,
			CountriesRepository countriesRepository, RegionsRepository regionsRepository,
			PaymentRegistriesRepository paymentRegistriesRepository, PaymentFormsRepository paymentFormsRepository,
			PricesRepository pricesRepository, CurrenciesRepository currenciesRepository,
			PriceTypeMapper priceTypeMapper, MessagesService messagesService)
		{
			InitializeComponent();
			Text = "Kontrahent";

			this.contractor = contractor;
			this.contractorsRepository = contractorsRepository;
			this.countriesRepository = countriesRepository;
			this.regionsRepository = regionsRepository;
			this.paymentFormsRepository = paymentFormsRepository;
			this.paymentRegistriesRepository = paymentRegistriesRepository;
			this.pricesRepository = pricesRepository;
			this.currenciesRepository = currenciesRepository;
			this.priceTypeMapper = priceTypeMapper;
			this.messagesService = messagesService;
			intitializeBinding();
		}

		private void ControlContractor_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemSave_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				ResponseResult<Contractor> _contractor;
				if (contractor.Id > 0)
					_contractor = contractorsRepository.Update(contractor);
				else
					_contractor = contractorsRepository.AddNew(contractor);

				if (_contractor.IsSucceed)
				{
					if (contractor.Id > 0)
					{
						var _contractors2 = contractorsRepository.Get(contractor.Id.Value);
						if (_contractors2.IsSucceed)
							contractor = _contractors2.Object;
					}
					else
						contractor = _contractor.Object;

					refresh();
					XtraMessageBox.Show("Zapisano zmiany");
				}
				else
					messagesService.AddMessage(_contractor);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemCancel_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				if (contractor.Id > 0)
				{
					var _contractors2 = contractorsRepository.Get(contractor.Id.Value);
					if (_contractors2.IsSucceed)
						contractor = _contractors2.Object;
					else
						messagesService.AddMessage(_contractors2);
				}
				else
				{
					contractor = new Contractor()
					{
						DefaultAddress = new ContractorDefaultAddress(),
						CorrespondenceAddress = new ContractorCorrespondenceAddress(),
						Contact = new ContractorContact(),
						PaymentRegistry = new PaymentRegistryBase(),
						PaymentForm = new PaymentFormBase()
					};
				}

				refresh();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void lookUpEditRegister_EditValueChanged(object sender, EventArgs e)
		{
			try
			{
				if (lookUpEditRegister.EditValue == DBNull.Value)
					return;

				var idRegistry = Convert.ToInt32(lookUpEditRegister.EditValue);
				if (idRegistry > 0)
				{

					var paymentForms = paymentFormsRepository.Get(idRegistry);
					if (paymentForms.IsSucceed)
					{
						lookUpEditPaymentForm.Properties.DataSource = paymentForms.Object;

						if (lookUpEditPaymentForm.EditValue == DBNull.Value)
							lookUpEditPaymentForm.EditValue = null;
						else
						{
							var idPayment = Convert.ToInt32(lookUpEditPaymentForm.EditValue);
							if (!paymentForms.Object.Exists(x => x.Id == idPayment))
								lookUpEditPaymentForm.EditValue = null;
						}
					}
					else
						messagesService.AddMessage(paymentForms);
				}
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}

		private void initialize()
		{
			var countries = countriesRepository.GetStatic();
			if (countries.IsSucceed)
			{
				lookUpEditCountry.Properties.DataSource = countries.Object?.Where(x => x.Active).ToList();
				lookUpEditCountry.Properties.DisplayMember = "Code";
				lookUpEditCountry.Properties.ValueMember = "Code";

				lookUpEditCountry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditCountry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));

				lookUpEditCCountry.Properties.DataSource = countries.Object?.Where(x => x.Active).ToList();
				lookUpEditCCountry.Properties.DisplayMember = "Code";
				lookUpEditCCountry.Properties.ValueMember = "Code";

				lookUpEditCCountry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditCCountry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(countries);

			var regions = regionsRepository.GetStatic();
			if (regions.IsSucceed)
			{
				lookUpEditProvince.Properties.DataSource = regions.Object?.Where(x => x.Active).ToList();
				lookUpEditProvince.Properties.DisplayMember = "Name";
				lookUpEditProvince.Properties.ValueMember = "Name";

				lookUpEditProvince.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));

				lookUpEditCProvince.Properties.DataSource = regions.Object?.Where(x => x.Active).ToList();
				lookUpEditCProvince.Properties.DisplayMember = "Name";
				lookUpEditCProvince.Properties.ValueMember = "Name";

				lookUpEditCProvince.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(regions);

			var paymentRegistries = paymentRegistriesRepository.GetStatic();
			if (paymentRegistries.IsSucceed)
			{
				lookUpEditRegister.Properties.DataSource = paymentRegistries.Object;
				lookUpEditRegister.Properties.DisplayMember = "Name";
				lookUpEditRegister.Properties.ValueMember = "Id";

				lookUpEditRegister.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(paymentRegistries);

			var priceTypes = pricesRepository.GetSalePriceTypesStatic();
			if (priceTypes.IsSucceed)
			{
				var models = priceTypeMapper.Map(priceTypes.Object);
				lookUpEditPriceType.Properties.DataSource = models;
				lookUpEditPriceType.Properties.DisplayMember = "Code";
				lookUpEditPriceType.Properties.ValueMember = "Type";

				lookUpEditPriceType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
			}
			else
				messagesService.AddMessage(priceTypes);

			var currencies = currenciesRepository.GetStatic();
			if (currencies.IsSucceed)
			{
				lookUpEditCurrency.Properties.DataSource = currencies.Object;
				lookUpEditCurrency.Properties.DisplayMember = "Shortcut";
				lookUpEditCurrency.Properties.ValueMember = "Shortcut";

				lookUpEditCurrency.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Shortcut", "Skrót"));
				lookUpEditCurrency.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(currencies);

			List<Tuple<enumPriceKind, string>> priceKinds = new List<Tuple<enumPriceKind, string>>();
			priceKinds.Add(new Tuple<enumPriceKind, string>(enumPriceKind.Undefined, enumPriceKind.Undefined.GetAttributeDescription<EnumPolishName>()));
			priceKinds.Add(new Tuple<enumPriceKind, string>(enumPriceKind.Gross, enumPriceKind.Gross.GetAttributeDescription<EnumPolishName>()));
			priceKinds.Add(new Tuple<enumPriceKind, string>(enumPriceKind.Net, enumPriceKind.Net.GetAttributeDescription<EnumPolishName>()));
			lookUpEditPriceKind.Properties.DataSource = priceKinds;
			lookUpEditPriceKind.Properties.DisplayMember = "Item2";
			lookUpEditPriceKind.Properties.ValueMember = "Item1";

			lookUpEditPriceKind.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Kod"));

			List<Tuple<enumContractorType, string>> contractorTypes = new List<Tuple<enumContractorType, string>>();
			contractorTypes.Add(new Tuple<enumContractorType, string>(enumContractorType.Person, enumContractorType.Person.GetAttributeDescription<EnumPolishName>()));
			contractorTypes.Add(new Tuple<enumContractorType, string>(enumContractorType.PersonFirm, enumContractorType.PersonFirm.GetAttributeDescription<EnumPolishName>()));
			contractorTypes.Add(new Tuple<enumContractorType, string>(enumContractorType.Firm, enumContractorType.Firm.GetAttributeDescription<EnumPolishName>()));
			lookUpEditType.Properties.DataSource = contractorTypes;
			lookUpEditType.Properties.DisplayMember = "Item2";
			lookUpEditType.Properties.ValueMember = "Item1";

			lookUpEditType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			lookUpEditPaymentForm.Properties.DisplayMember = "Name";
			lookUpEditPaymentForm.Properties.ValueMember = "Id";
			lookUpEditPaymentForm.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			lookUpEditPaymentForm.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Days", "Dni"));

			if (contractor.Id > 0)
			{
				textEditCode.ReadOnly = true;
				lookUpEditType.ReadOnly = true;
			}
		}
		private void intitializeBinding()
		{
			textEditCode.DataBindings.Add("EditValue", contractor, nameof(contractor.Code), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditName.DataBindings.Add("EditValue", contractor, nameof(contractor.Name), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditActive.DataBindings.Add("EditValue", contractor, nameof(contractor.Active), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditType.DataBindings.Add("EditValue", contractor, nameof(contractor.Type), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditCountry.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultAddress) + "." + nameof(contractor.DefaultAddress.Country), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditProvince.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultAddress) + "." + nameof(contractor.DefaultAddress.Province), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditCity.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultAddress) + "." + nameof(contractor.DefaultAddress.City), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPostCode.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultAddress) + "." + nameof(contractor.DefaultAddress.PostCode), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditStreet.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultAddress) + "." + nameof(contractor.DefaultAddress.Street), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditHouseNo.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultAddress) + "." + nameof(contractor.DefaultAddress.HouseNo), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditApartmentNo.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultAddress) + "." + nameof(contractor.DefaultAddress.ApartmentNo), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditCCountry.DataBindings.Add("EditValue", contractor, nameof(contractor.CorrespondenceAddress) + "." + nameof(contractor.CorrespondenceAddress.Country), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditCProvince.DataBindings.Add("EditValue", contractor, nameof(contractor.CorrespondenceAddress) + "." + nameof(contractor.CorrespondenceAddress.Province), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditCCity.DataBindings.Add("EditValue", contractor, nameof(contractor.CorrespondenceAddress) + "." + nameof(contractor.CorrespondenceAddress.City), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditCPostCode.DataBindings.Add("EditValue", contractor, nameof(contractor.CorrespondenceAddress) + "." + nameof(contractor.CorrespondenceAddress.PostCode), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditCStreet.DataBindings.Add("EditValue", contractor, nameof(contractor.CorrespondenceAddress) + "." + nameof(contractor.CorrespondenceAddress.Street), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditCHouseNo.DataBindings.Add("EditValue", contractor, nameof(contractor.CorrespondenceAddress) + "." + nameof(contractor.CorrespondenceAddress.HouseNo), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditCApartmentNo.DataBindings.Add("EditValue", contractor, nameof(contractor.CorrespondenceAddress) + "." + nameof(contractor.CorrespondenceAddress.ApartmentNo), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditPesel.DataBindings.Add("EditValue", contractor, nameof(contractor.Pesel), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditNIP.DataBindings.Add("EditValue", contractor, nameof(contractor.NIP), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditRegon.DataBindings.Add("EditValue", contractor, nameof(contractor.Regon), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditVIES.DataBindings.Add("EditValue", contractor, nameof(contractor.Vies), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditVAT.DataBindings.Add("EditValue", contractor, nameof(contractor.VATTaxPayer), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditRegister.DataBindings.Add("EditValue", contractor, nameof(contractor.PaymentRegistry) + "." + nameof(contractor.PaymentRegistry.Id), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPaymentForm.DataBindings.Add("EditValue", contractor, nameof(contractor.PaymentForm) + "." + nameof(contractor.PaymentForm.Id), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPriceType.DataBindings.Add("EditValue", contractor, nameof(contractor.PriceType), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPriceKind.DataBindings.Add("EditValue", contractor, nameof(contractor.PriceKind), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditNegotation.DataBindings.Add("EditValue", contractor, nameof(contractor.PriceNegotiation), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditLimit.DataBindings.Add("EditValue", contractor, nameof(contractor.CreditLimit), true, DataSourceUpdateMode.OnPropertyChanged);
			calcEditCredit.DataBindings.Add("EditValue", contractor, nameof(contractor.MaxCreditValue), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditCurrency.DataBindings.Add("EditValue", contractor, nameof(contractor.CreditCurrency), true, DataSourceUpdateMode.OnPropertyChanged);
			calcEditDiscount.DataBindings.Add("EditValue", contractor, nameof(contractor.DefaultDiscountPercent), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditFirstName.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Name), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditSurname.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Surname), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPhone1.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Phone1), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPhone2.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Phone2), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditFax.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Fax), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditEmail.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Email), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditWWW.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.WWW), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditFacebook.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Facebook), true, DataSourceUpdateMode.OnPropertyChanged);
		}
		private void refresh()
		{
			textEditCode.DataBindings.Clear();
			textEditName.DataBindings.Clear();
			checkEditActive.DataBindings.Clear();
			lookUpEditType.DataBindings.Clear();

			lookUpEditCountry.DataBindings.Clear();
			lookUpEditProvince.DataBindings.Clear();
			textEditCity.DataBindings.Clear();
			textEditPostCode.DataBindings.Clear();
			textEditStreet.DataBindings.Clear();
			textEditHouseNo.DataBindings.Clear();
			textEditApartmentNo.DataBindings.Clear();

			lookUpEditCCountry.DataBindings.Clear();
			lookUpEditCProvince.DataBindings.Clear();
			textEditCCity.DataBindings.Clear();
			textEditCPostCode.DataBindings.Clear();
			textEditCStreet.DataBindings.Clear();
			textEditCHouseNo.DataBindings.Clear();
			textEditCApartmentNo.DataBindings.Clear();

			textEditPesel.DataBindings.Clear();
			textEditNIP.DataBindings.Clear();
			textEditRegon.DataBindings.Clear();
			checkEditVIES.DataBindings.Clear();
			checkEditVAT.DataBindings.Clear();

			lookUpEditRegister.DataBindings.Clear();
			lookUpEditPaymentForm.DataBindings.Clear();
			lookUpEditPriceType.DataBindings.Clear();
			lookUpEditPriceKind.DataBindings.Clear();
			checkEditNegotation.DataBindings.Clear();
			checkEditLimit.DataBindings.Clear();
			calcEditCredit.DataBindings.Clear();
			lookUpEditCurrency.DataBindings.Clear();
			calcEditDiscount.DataBindings.Clear();

			textEditFirstName.DataBindings.Clear();
			textEditSurname.DataBindings.Clear();
			textEditPhone1.DataBindings.Clear();
			textEditPhone2.DataBindings.Clear();
			textEditFax.DataBindings.Clear();
			textEditEmail.DataBindings.Clear();
			textEditWWW.DataBindings.Clear();
			textEditFacebook.DataBindings.Clear();
			intitializeBinding();
		}
	}
}
