﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Unity;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Interface.Contractors.ViewModels;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Demo.Model;
using DevExpress.XtraGrid.Views.Grid;
using Unity.Resolution;
using DevExpress.XtraEditors;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using Sage.PL.WebAPI.Interface.Common.ViewModels;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlContractorsIndex : ManagedControl
	{
		public override Guid Identifier => new Guid("421532D7-FB0B-42CF-AA51-5E6D7D917033");

		private readonly IUnityContainer unityContainer;
		private readonly GridControlService gridControlService;
		private readonly DockManagerService dockManagerService;
		private readonly MessagesService messagesService;
		private readonly ContractorsRepository contractorsRepository;
		private readonly ContractorMapper contractorMapper;

		private List<ContractorModel> contractors;

		public ControlContractorsIndex(IUnityContainer unityContainer, GridControlService gridControlService, DockManagerService dockManagerService, MessagesService messagesService,
			ContractorsRepository contractorsRepository, ContractorMapper contractorMapper)
		{
			this.unityContainer = unityContainer;
			this.gridControlService = gridControlService;
			this.dockManagerService = dockManagerService;
			this.messagesService = messagesService;
			this.contractorsRepository = contractorsRepository;
			this.contractorMapper = contractorMapper;
			InitializeComponent();
			Text = "Kartoteka kontrahentów";
		}

		private void ControlContractorsIndex_Load(object sender, EventArgs e)
		{
			try
			{
				barStaticItemGuid.Caption = unityContainer.Resolve<string>("SessionGuid");
				loadData();
				gridViewContractors.PopulateColumns();

				gridControlService.SetColumnReadOnly(ref gridViewContractors, true);
				gridControlService.SetLocalization(ref gridViewContractors, localizations);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemEdit_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				int handle = gridViewContractors.FocusedRowHandle;
				if (handle < 0)
					return;

				var indexContractor = gridViewContractors.GetRow(handle) as ContractorModel;
				var response = contractorsRepository.Get(indexContractor.Id);
				if (response.IsSucceed)
				{
					var control = unityContainer.Resolve<ControlContractor>(new ParameterOverride("contractor", response.Object));
					dockManagerService.OpenPanel(control);
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemNew_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				Contractor contractor = new Contractor()
				{
					DefaultAddress = new ContractorDefaultAddress(),
					CorrespondenceAddress = new ContractorCorrespondenceAddress(),
					Contact = new ContractorContact(),
					PaymentRegistry = new PaymentRegistryBase(),
					PaymentForm = new PaymentFormBase()
				};

				var control = unityContainer.Resolve<ControlContractor>(new ParameterOverride("contractor", contractor));
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemRefresh_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));
				loadData();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void gridViewContractors_DoubleClick(object sender, EventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				GridView gridView = sender as GridView;
				if (gridView == null)
					return;

				int handle = gridView.FocusedRowHandle;
				if (handle < 0)
					return;

				var indexContractor = gridView.GetRow(handle) as ContractorModel;
				var response = contractorsRepository.Get(indexContractor.Id);
				if (response.IsSucceed)
				{
					var control = unityContainer.Resolve<ControlContractor>(new ParameterOverride("contractor", response.Object));
					dockManagerService.OpenPanel(control);
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void loadData()
		{
			var response = contractorsRepository.Get();
			if (response.IsSucceed)
			{
				var _contractors = response.Object;
				contractors = contractorMapper.Map(_contractors);
				gridControlContractors.DataSource = contractors;
				gridControlContractors.RefreshDataSource();
			}
			else
				messagesService.AddMessage(response);
		}
		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "Code", "Kod" },
			{ "Name", "Nazwa" },
			{ "Type", "Typ" },
			{ "Active", "Aktywny" },
			{ "PriceType", "Typ ceny" },
			{ "PriceKind", "Rodzaj ceny" },
			{ "PaymentFormOid", "Domyślna forma płatności" },
			{ "PaymentRegistryOid", "Domyślny rejestr płatności" },
			{ "Place", "Miejscowość" },
		};
	}
}
