﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Demo.Extensions;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using DevExpress.XtraEditors;
using Sage.PL.WebAPI.Demo.Forms;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Interface.FKF.Contractors.ViewModels;
using Sage.PL.WebAPI.Demo.DAL;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlFKContractor : ManagedControl
	{
		public override Guid Identifier => new Guid("A6212BCA-C857-4AC0-8117-1176368244B6");

		private Contractor contractor;
		private readonly FKContractorsRepository contractorsRepository;
		private readonly FKCountriesRepository countriesRepository;
		private readonly FKRegionsRepository regionsRepository;
		private readonly MessagesService messagesService;

		public ControlFKContractor(Contractor contractor, FKContractorsRepository contractorsRepository,
			FKCountriesRepository countriesRepository, FKRegionsRepository regionsRepository,
			MessagesService messagesService)
		{
			InitializeComponent();
			Text = "Kontrahent FK";

			this.contractor = contractor;
			this.contractorsRepository = contractorsRepository;
			this.countriesRepository = countriesRepository;
			this.regionsRepository = regionsRepository;
			this.messagesService = messagesService;
			intitializeBinding();
		}

		private void ControlContractor_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemSave_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				ResponseResult<Contractor> _contractor;
				if (contractor.Id > 0)
					_contractor = contractorsRepository.Update(contractor);
				else
					_contractor = contractorsRepository.AddNew(contractor);

				if (_contractor.IsSucceed)
				{
					if (contractor.Id > 0)
					{
						var _contractors2 = contractorsRepository.GetById(contractor.Id.Value);
						if (_contractors2.IsSucceed)
							contractor = _contractors2.Object;
					}
					else
						contractor = _contractor.Object;

					refresh();
					XtraMessageBox.Show("Zapisano zmiany");
				}
				else
					messagesService.AddMessage(_contractor);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemCancel_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				if (contractor.Id > 0)
				{
					var _contractors2 = contractorsRepository.GetById(contractor.Id.Value);
					if (_contractors2.IsSucceed)
						contractor = _contractors2.Object;
					else
						messagesService.AddMessage(_contractors2);
				}
				else
				{
					contractor = new Contractor()
					{
						Address = new ContractorAddress(),
						Contact = new ContractorContact(),
					};
				}

				refresh();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void initialize()
		{
			var countries = countriesRepository.GetStatic();
			if (countries.IsSucceed)
			{
				lookUpEditCountry.Properties.DataSource = countries.Object?.Where(x => x.Active).ToList();
				lookUpEditCountry.Properties.DisplayMember = "Code";
				lookUpEditCountry.Properties.ValueMember = "Code";

				lookUpEditCountry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditCountry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(countries);

			var regions = regionsRepository.GetStatic();
			if (regions.IsSucceed)
			{
				lookUpEditProvince.Properties.DataSource = regions.Object?.Where(x => x.Active).ToList();
				lookUpEditProvince.Properties.DisplayMember = "Name";
				lookUpEditProvince.Properties.ValueMember = "Name";

				lookUpEditProvince.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(regions);

			List<Tuple<enumContractorType, string>> contractorTypes = new List<Tuple<enumContractorType, string>>();
			contractorTypes.Add(new Tuple<enumContractorType, string>(enumContractorType.Person, enumContractorType.Person.GetAttributeDescription<EnumPolishName>()));
			contractorTypes.Add(new Tuple<enumContractorType, string>(enumContractorType.PersonFirm, enumContractorType.PersonFirm.GetAttributeDescription<EnumPolishName>()));
			contractorTypes.Add(new Tuple<enumContractorType, string>(enumContractorType.Firm, enumContractorType.Firm.GetAttributeDescription<EnumPolishName>()));
			lookUpEditType.Properties.DataSource = contractorTypes;
			lookUpEditType.Properties.DisplayMember = "Item2";
			lookUpEditType.Properties.ValueMember = "Item1";

			lookUpEditType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			if (contractor.Id > 0)
			{
				textEditCode.ReadOnly = true;
				lookUpEditType.ReadOnly = true;
			}
		}
		private void intitializeBinding()
		{
			textEditCode.DataBindings.Add("EditValue", contractor, nameof(contractor.Code), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditName.DataBindings.Add("EditValue", contractor, nameof(contractor.Name), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditActive.DataBindings.Add("EditValue", contractor, nameof(contractor.Active), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditType.DataBindings.Add("EditValue", contractor, nameof(contractor.Type), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPosition.DataBindings.Add("EditValue", contractor, nameof(contractor.Position), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditCountry.DataBindings.Add("EditValue", contractor, nameof(contractor.Address) + "." + nameof(contractor.Address.Country), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditProvince.DataBindings.Add("EditValue", contractor, nameof(contractor.Address) + "." + nameof(contractor.Address.Province), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditCity.DataBindings.Add("EditValue", contractor, nameof(contractor.Address) + "." + nameof(contractor.Address.City), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPostCode.DataBindings.Add("EditValue", contractor, nameof(contractor.Address) + "." + nameof(contractor.Address.PostCode), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditStreet.DataBindings.Add("EditValue", contractor, nameof(contractor.Address) + "." + nameof(contractor.Address.Street), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditHouseNo.DataBindings.Add("EditValue", contractor, nameof(contractor.Address) + "." + nameof(contractor.Address.HouseNo), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditApartmentNo.DataBindings.Add("EditValue", contractor, nameof(contractor.Address) + "." + nameof(contractor.Address.ApartmentNo), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditPesel.DataBindings.Add("EditValue", contractor, nameof(contractor.Pesel), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditNIP.DataBindings.Add("EditValue", contractor, nameof(contractor.NIP), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditRegon.DataBindings.Add("EditValue", contractor, nameof(contractor.Regon), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditVIES.DataBindings.Add("EditValue", contractor, nameof(contractor.Vies), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditVAT.DataBindings.Add("EditValue", contractor, nameof(contractor.VATTaxPayer), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditFirstName.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Name), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditSurname.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Surname), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPhone1.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Phone1), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPhone2.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Phone2), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditFax.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Fax), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditEmail.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Email), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditWWW.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.WWW), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditFacebook.DataBindings.Add("EditValue", contractor, nameof(contractor.Contact) + "." + nameof(contractor.Contact.Facebook), true, DataSourceUpdateMode.OnPropertyChanged);
		}
		private void refresh()
		{
			textEditCode.DataBindings.Clear();
			textEditName.DataBindings.Clear();
			checkEditActive.DataBindings.Clear();
			lookUpEditType.DataBindings.Clear();
			textEditPosition.DataBindings.Clear();

			lookUpEditCountry.DataBindings.Clear();
			lookUpEditProvince.DataBindings.Clear();
			textEditCity.DataBindings.Clear();
			textEditPostCode.DataBindings.Clear();
			textEditStreet.DataBindings.Clear();
			textEditHouseNo.DataBindings.Clear();
			textEditApartmentNo.DataBindings.Clear();

			textEditPesel.DataBindings.Clear();
			textEditNIP.DataBindings.Clear();
			textEditRegon.DataBindings.Clear();
			checkEditVIES.DataBindings.Clear();
			checkEditVAT.DataBindings.Clear();

			textEditFirstName.DataBindings.Clear();
			textEditSurname.DataBindings.Clear();
			textEditPhone1.DataBindings.Clear();
			textEditPhone2.DataBindings.Clear();
			textEditFax.DataBindings.Clear();
			textEditEmail.DataBindings.Clear();
			textEditWWW.DataBindings.Clear();
			textEditFacebook.DataBindings.Clear();
			intitializeBinding();
		}
	}
}

