﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using DevExpress.XtraEditors;
using System.Net.Http;
using Sage.PL.WebAPI.Interface.Contractors.Interfaces;
using System.Threading;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Unity;
using Unity.Resolution;
using System.Diagnostics;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Forms;
using System.Security.Cryptography;
using Sage.PL.WebAPI.Interface.FKF.Contractors.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Contractors.Interfaces;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlFKContractorTest : ManagedControl, ICanClose
	{
		private readonly IUnityContainer unityContainer;
		private readonly HttpClient originalHttpClient;
		private readonly SessionRepository sessionRepository;
		private readonly TestRepository testRepository;
		private readonly MessagesService messagesService;
		private readonly FKContractorsRepository contractorsRepository;

		private List<ContractorListElement> contractors;
		private TimeSpan sumTime;
		private bool edit = true;

		public FormMain FormMain { get; set; }

		public override Guid Identifier => new Guid("B7767213-B25A-426B-BC30-E44A646B630D");

		public ControlFKContractorTest(IUnityContainer unityContainer,
			HttpClient originalHttpClient, SessionRepository sessionRepository, TestRepository testRepository, MessagesService messagesService,
			FKContractorsRepository contractorsRepository)
		{
			InitializeComponent();
			initialize();
			setEdit(true);
			Text = "Test kontrahenta";

			this.unityContainer = unityContainer;
			this.originalHttpClient = originalHttpClient;
			this.sessionRepository = sessionRepository;
			this.testRepository = testRepository;
			this.messagesService = messagesService;
			this.contractorsRepository = contractorsRepository;

			progressBarControl.Properties.Step = 1;
			progressBarControl.Properties.PercentView = true;
			progressBarControl.Properties.Minimum = 0;
		}

		private async void barButtonItemStart_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				int type = Convert.ToInt32(lookUpEditType.EditValue);
				int threadNumber = Convert.ToInt32(spinEditThreads.EditValue);
				int opeartions = Convert.ToInt32(spinEditOperationsInThreads.EditValue);
				labelControlEndDate.Text = "TBA";
				labelControlTime.Text = "TBA";
				labelControlSumTime.Text = "TBA";
				labelControlMean.Text = "TBA";

				progressBarControl.Position = 0;
				progressBarControl.Properties.Maximum = threadNumber * opeartions;
				setEdit(false);

				if (type == 0)
				{
					richTextBoxResults.Text = "Guid sesji\tId kontrahenta\tCzas\tStatus" + Environment.NewLine;
					var _contractors = contractorsRepository.Get();
					if (!_contractors.IsSucceed)
					{
						messagesService.AddMessage(_contractors);
						return;
					}

					contractors = _contractors.Object;

					List<Task> tasks = new List<Task>();
					List<string> sessions = new List<string>();
					for (int i = 0; i < threadNumber; i++)
					{
						var _result = sessionRepository.OpenNewSession($"Test kontrahenta {i + 1}");
						if (_result.IsSucceed)
						{
							Task task = new Task(() => getTest(opeartions, _result.Object));
							tasks.Add(task);
							sessions.Add(_result.Object);
						}
						else
						{
							messagesService.AddMessage(_result);
							return;
						}
					}

					sumTime = new TimeSpan();
					Stopwatch global = new Stopwatch();
					labelControlStartDate.Text = DateTime.Now.ToString();

					global.Start();
					foreach (var task in tasks)
						task.Start();

					await Task.WhenAll(tasks);
					global.Stop();
					labelControlTime.Text = global.Elapsed.ToString();
					labelControlEndDate.Text = DateTime.Now.ToString();
					labelControlSumTime.Text = sumTime.ToString();
					labelControlMean.Text = new TimeSpan(sumTime.Ticks / (threadNumber * opeartions)).ToString();

					foreach (var session in sessions)
						testRepository.CloseSession(session);
				}
				else
				{
					richTextBoxResults.Text = "Guid sesji\tKod kontrahenta\tCzas\tStatus" + Environment.NewLine;

					List<Task> tasks = new List<Task>();
					List<string> sessions = new List<string>();
					for (int i = 0; i < threadNumber; i++)
					{
						var _result = sessionRepository.OpenNewSession($"Test kontrahenta {i + 1}");
						if (_result.IsSucceed)
						{
							Task task = new Task(() => createTest(opeartions, _result.Object));
							tasks.Add(task);
							sessions.Add(_result.Object);
						}
						else
						{
							messagesService.AddMessage(_result);
							return;
						}
					}

					sumTime = new TimeSpan();
					Stopwatch global = new Stopwatch();
					labelControlStartDate.Text = DateTime.Now.ToString();

					global.Start();
					foreach (var task in tasks)
						task.Start();

					await Task.WhenAll(tasks);
					global.Stop();
					labelControlTime.Text = global.Elapsed.ToString();
					labelControlEndDate.Text = DateTime.Now.ToString();
					labelControlSumTime.Text = sumTime.ToString();
					labelControlMean.Text = new TimeSpan(sumTime.Ticks / (threadNumber * opeartions)).ToString();

					foreach (var session in sessions)
						testRepository.CloseSession(session);
				}
			}
			catch (Exception exc)
			{
				XtraMessageBox.Show(exc.ToString());
			}
			finally
			{
				setEdit(true);
			}
		}
		private void getTest(int operations, string guidSession)
		{
			try
			{
				int threadId = Thread.CurrentThread.ManagedThreadId;
				Random random = new Random(threadId);
				int maxValue = contractors.Count() - 1;

				using (HttpClient httpClient = new HttpClient())
				{
					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					IFKContractorsController contractorsHandler = unityContainer.Resolve<IFKContractorsController>(
						new ParameterOverride("httpClient", httpClient),
						new ParameterOverride("sessionGuid", guidSession));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					for (int i = 0; i < operations; i++)
					{
						var index = random.Next(0, maxValue);
						int id = contractors[index].Id;

						stopwatch.Restart();
						var result = contractorsHandler.GetById(id) as ResponseMessageResult;
						var succeed = result.Response.IsSuccessStatusCode;
						stopwatch.Stop();

						richTextBoxResults.Invoke((MethodInvoker)delegate
						{
							richTextBoxResults.Text += string.Format("{0}\t{1}\t{2}\t{3}{4}",
								guidSession,
								id,
								stopwatch.Elapsed,
								succeed,
								Environment.NewLine);

							sumTime = sumTime.Add(stopwatch.Elapsed);
						});

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}
				}
			}
			catch
			{ }
		}
		private void createTest(int operations, string guidSession)
		{
			try
			{
				using (HttpClient httpClient = new HttpClient())
				{
					int threadId = Thread.CurrentThread.ManagedThreadId;
					Random random = new Random(threadId);

					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					IFKContractorsController contractorsHandler = unityContainer.Resolve<IFKContractorsController>(
						new ParameterOverride("httpClient", httpClient),
						new ParameterOverride("sessionGuid", guidSession));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					for (int i = 0; i < operations; i++)
					{
						Contractor contractor = new Contractor();
						Guid guid = Guid.NewGuid();
						var stringGuid = guid.ToString().Replace("-", "") + "_" + threadId;
						contractor.Code = stringGuid;
						contractor.Name = stringGuid;
						contractor.Active = true;

						stopwatch.Restart();
						var result = contractorsHandler.AddNew(contractor) as ResponseMessageResult;
						var succeed = result.Response.IsSuccessStatusCode;
						stopwatch.Stop();

						richTextBoxResults.Invoke((MethodInvoker)delegate
						{
							richTextBoxResults.Text += string.Format("{0}\t{1}\t{2}\t{3}{4}",
								guidSession,
								stringGuid,
								stopwatch.Elapsed,
								succeed,
								Environment.NewLine);

							sumTime = sumTime.Add(stopwatch.Elapsed);
						});

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}
				}
			}
			catch
			{ }
		}


		private void initialize()
		{
			List<Tuple<int, string>> types = new List<Tuple<int, string>>();
			types.Add(new Tuple<int, string>(0, "Pobieranie"));
			types.Add(new Tuple<int, string>(1, "Dodawanie"));
			lookUpEditType.Properties.DataSource = types;
			lookUpEditType.Properties.DisplayMember = "Item2";
			lookUpEditType.Properties.ValueMember = "Item1";

			lookUpEditType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));
			lookUpEditType.EditValue = 0;

			spinEditOperationsInThreads.Properties.MaxValue = int.MaxValue;
			spinEditThreads.Properties.MaxValue = int.MaxValue;
		}
		private void setEdit(bool mode)
		{
			lookUpEditType.Enabled = mode;
			spinEditOperationsInThreads.Enabled = mode;
			spinEditThreads.Enabled = mode;
			FormMain?.SetEdit(mode);
			edit = mode;
		}

		public bool CanClose()
		{
			return edit;
		}
	}
}
