﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using DevExpress.XtraEditors;
using Sage.PL.WebAPI.Demo.ViewModels;
using Sage.PL.WebAPI.Demo.DAL.Repositories;

using Sage.PL.WebAPI.Demo.Mappers;
using DevExpress.XtraGrid.Views.Grid;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlFKDocument : ManagedControl
	{
		public override Guid Identifier => new Guid("849D3AB6-7507-4A3C-BD8D-BD378D23F7D4");

		private readonly FKDocumentsRepository documentsRepository;
		private readonly GridControlService gridControlService;
		private readonly FKDocumentMapper documentMapper;
		private readonly MessagesService messagesService;
		private FKDocumentViewModel document;
		public int YearId { get; set; }

		public ControlFKDocument(FKDocumentsRepository documentsRepository, GridControlService gridControlService, FKDocumentMapper documentMapper, MessagesService messagesService)
		{
			InitializeComponent();
			Text = "Dokument prosty";

			document = new FKDocumentViewModel()
			{
				IssueDate = DateTime.Now,
				DocumentDate = DateTime.Now,
				OperationDate = DateTime.Now,
				Records = new List<FKDocumentRecordViewModel>()
			};

			this.documentsRepository = documentsRepository;
			this.gridControlService = gridControlService;
			this.documentMapper = documentMapper;
			this.messagesService = messagesService;
			intitializeBinding();
		}

		private void ControlForeignOrder_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemSave_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				gridViewRecords.PostEditor();
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var documentIssue = documentMapper.Map(document, YearId);
				var _document = documentsRepository.AddNew(documentIssue);
				if (_document.IsSucceed)
					XtraMessageBox.Show("Zapisano nowy dokument");
				else
					messagesService.AddMessage(_document);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonAddPosition_Click(object sender, EventArgs e)
		{
			try
			{
				int no = document.Records.GroupBy(x => x.No).Count();

				document.Records.Add(new FKDocumentRecordViewModel() { No = no, SplitNo = 0, Side = Interface.Enums.enumSideType.Debit });
				document.Records.Add(new FKDocumentRecordViewModel() { No = no, SplitNo = 1, Side = Interface.Enums.enumSideType.Credit });

				gridControlRecords.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonDelete_Click(object sender, EventArgs e)
		{
			try
			{
				int handle = gridViewRecords.FocusedRowHandle;
				if (handle < 0)
					return;

				FKDocumentRecordViewModel record = gridViewRecords.GetRow(handle) as FKDocumentRecordViewModel;
				if (record == null)
					return;

				var position = document.Records.Where(x => x.No == record.No).ToList();
				foreach (var _position in position)
					document.Records.Remove(_position);

				var group = document.Records.GroupBy(x => x.No);
				int no = 0;
				foreach (var _group in group)
				{
					foreach (var _record in _group)
						_record.No = no;
					no++;
				}

				gridControlRecords.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonAddSplit_Click(object sender, EventArgs e)
		{
			try
			{
				int handle = gridViewRecords.FocusedRowHandle;
				if (handle < 0)
					return;

				FKDocumentRecordViewModel record = gridViewRecords.GetRow(handle) as FKDocumentRecordViewModel;
				if (record == null)
					return;

				var index = document.Records.IndexOf(document.Records.Where(x => x.No == record.No).Last()) + 1;
				int splitNo = document.Records.Where(x => x.No == record.No).Max(x => x.SplitNo) + 1;
				document.Records.Insert(index, new FKDocumentRecordViewModel() { No = record.No, SplitNo = splitNo });
				gridControlRecords.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonRemoveSplit_Click(object sender, EventArgs e)
		{
			try
			{
				int handle = gridViewRecords.FocusedRowHandle;
				if (handle < 0)
					return;

				FKDocumentRecordViewModel record = gridViewRecords.GetRow(handle) as FKDocumentRecordViewModel;
				if (record == null)
					return;

				if (record.SplitNo == 0 || record.SplitNo == 1)
				{
					XtraMessageBox.Show("Nie można usunąć pierwszych dwóch rozbić w zapisie");
					return;
				}

				document.Records.Remove(record);

				int splitNo = 0;
				foreach (var _record in document.Records.Where(x => x.No == record.No))
				{
					_record.SplitNo = splitNo;
					splitNo++;
				}

				gridControlRecords.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void gridViewRecords_ShowingEditor(object sender, CancelEventArgs e)
		{
			try
			{
				GridView gridView = sender as GridView;
				if (gridView == null)
					return;

				var handle = gridView.FocusedRowHandle;
				if (handle < 0)
					return;

				FKDocumentRecordViewModel record = gridView.GetRow(handle) as FKDocumentRecordViewModel;
				if (record == null)
					return;

				var column = gridView.FocusedColumn;

				if ((record.SplitNo == 0 || record.SplitNo == 1) && nameof(FKDocumentRecordViewModel.Side).Equals(column.FieldName))
					e.Cancel = true;
			}
			finally { }
		}

		private void initialize()
		{
			var types = documentsRepository.GetDocumentTypes(YearId);
			if (types.IsSucceed)
			{
				lookUpEditDocumentSerie.Properties.DataSource = types.Object?.Where(x => x.Character == Interface.Enums.enumFKDocumentCharacter.DP).ToList();
				lookUpEditDocumentSerie.Properties.DisplayMember = "Code";
				lookUpEditDocumentSerie.Properties.ValueMember = "Code";

				lookUpEditDocumentSerie.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditDocumentSerie.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(types);

			gridControlRecords.DataSource = document.Records;
			gridControlRecords.RefreshDataSource();

			gridViewRecords.PopulateColumns();
			gridViewRecords.OptionsView.ColumnAutoWidth = false;

			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.SplitNo)].OptionsColumn.AllowEdit = false;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.SplitNo)].OptionsColumn.ReadOnly = true;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.No)].OptionsColumn.AllowEdit = false;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.No)].OptionsColumn.ReadOnly = true;

			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.No)].Width = 75;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.SplitNo)].Width = 75;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Side)].Width = 100;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Synthetic)].Width = 100;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Analytic1)].Width = 100;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Analytic2)].Width = 100;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Analytic3)].Width = 100;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Analytic4)].Width = 100;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Analytic5)].Width = 100;
			gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.ValuePLN)].Width = 100;

			gridViewRecords.AutoFillColumn = gridViewRecords.Columns[nameof(FKDocumentRecordViewModel.Description)];

			gridControlService.SetLocalization(ref gridViewRecords, localizations);
		}
		private void intitializeBinding()
		{
			dateEditIssueDate.DataBindings.Add("EditValue", document, nameof(document.IssueDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditDocumentDate.DataBindings.Add("EditValue", document, nameof(document.DocumentDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditOperationDate.DataBindings.Add("EditValue", document, nameof(document.OperationDate), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditDocumentSerie.DataBindings.Add("EditValue", document, nameof(document.TypeCode), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditDocumentNumber.DataBindings.Add("EditValue", document, nameof(document.DocumentNumber), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditContent.DataBindings.Add("EditValue", document, nameof(document.Content), true, DataSourceUpdateMode.OnPropertyChanged);
		}
		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "SplitNo", "Rozbicie" },
			{ "No", "Pozycja" },
			{ "Synthetic", "Syntetyk" },
			{ "Analytic1", "Analityk 1" },
			{ "Analytic2", "Analityk 2" },
			{ "Analytic3", "Analityk 3" },
			{ "Analytic4", "Analityk 4" },
			{ "Analytic5", "Analityk 5" },
			{ "Side", "Strona" },
			{ "Description", "Opis" },
			{ "ValuePLN", "Wartość" },
		};
	}
}
