﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using DevExpress.XtraEditors;
using System.Net.Http;
using Sage.PL.WebAPI.Interface.Contractors.Interfaces;
using System.Threading;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Unity;
using Unity.Resolution;
using System.Diagnostics;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Forms;
using System.Security.Cryptography;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.FKF.Documents.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Documents.Interfaces;
using Sage.PL.WebAPI.Interface.FKF.AccountsChart.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Documents.ViewModels.Issue.Simple;
using Sage.PL.WebAPI.Interface.FKF.Documents.ViewModels.Issue;
using Sage.PL.WebAPI.Interface.Enums;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlFKDocumentTest : ManagedControl, ICanClose
	{
		private readonly IUnityContainer unityContainer;
		private readonly MessagesService messagesService;
		private readonly HttpClient originalHttpClient;
		private readonly SessionRepository sessionRepository;
		private readonly TestRepository testRepository;
		private readonly FKYearsRepository yearsRepository;
		private readonly FKDocumentsRepository documentsRepository;
		private readonly FKAccountsChartRepository accountsChartRepository;

		private int yearId;
		private string documentType;
		private List<DocumentListElement> documents;
		private List<AccountChartElementSimple> accountCharts;

		private TimeSpan sumTime;
		private bool edit = true;

		public FormMain FormMain { get; set; }

		public override Guid Identifier => new Guid("4A307AD7-6491-4DE2-AB7D-47230D96BD52");

		public ControlFKDocumentTest(IUnityContainer unityContainer, MessagesService messagesService,
			HttpClient originalHttpClient, SessionRepository sessionRepository, TestRepository testRepository, FKYearsRepository yearsRepository,
			FKDocumentsRepository documentsRepository, FKAccountsChartRepository accountsChartRepository)
		{
			InitializeComponent();
			initialize();
			setEdit(true);
			Text = "Test dokumentu";

			this.unityContainer = unityContainer;
			this.messagesService = messagesService;
			this.originalHttpClient = originalHttpClient;
			this.sessionRepository = sessionRepository;
			this.testRepository = testRepository;
			this.yearsRepository = yearsRepository;
			this.documentsRepository = documentsRepository;
			this.accountsChartRepository = accountsChartRepository;
			progressBarControl.Properties.Step = 1;
			progressBarControl.Properties.PercentView = true;
			progressBarControl.Properties.Minimum = 0;
		}

		private async void barButtonItemStart_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				int type = Convert.ToInt32(lookUpEditType.EditValue);
				int threadNumber = Convert.ToInt32(spinEditThreads.EditValue);
				int opeartions = Convert.ToInt32(spinEditOperationsInThreads.EditValue);
				labelControlEndDate.Text = "TBA";
				labelControlTime.Text = "TBA";
				labelControlSumTime.Text = "TBA";
				labelControlMean.Text = "TBA";

				progressBarControl.Position = 0;
				progressBarControl.Properties.Maximum = threadNumber * opeartions;
				setEdit(false);

				if (type == 0)
				{
					richTextBoxResults.Text = "Guid sesji\tId dokumentu\tCzas\tStatus" + Environment.NewLine;

					var _years = yearsRepository.GetStatic();
					if (!_years.IsSucceed)
					{
						messagesService.AddMessage(_years);
						return;
					}
					yearId = _years.Object.LastOrDefault().Id;

					var _documents = documentsRepository.GetFromBuffer(yearId);
					if (!_documents.IsSucceed)
					{
						messagesService.AddMessage(_documents);
						return;
					}
					documents = _documents.Object;

					List<Task> tasks = new List<Task>();
					List<string> sessions = new List<string>();
					for (int i = 0; i < threadNumber; i++)
					{
						var _result = sessionRepository.OpenNewSession($"Test dokumentu {i + 1}");
						if (_result.IsSucceed)
						{
							Task task = new Task(() => getTest(opeartions, _result.Object));
							tasks.Add(task);
							sessions.Add(_result.Object);
						}
						else
						{
							messagesService.AddMessage(_result);
							return;
						}
					}

					sumTime = new TimeSpan();
					Stopwatch global = new Stopwatch();
					labelControlStartDate.Text = DateTime.Now.ToString();

					global.Start();
					foreach (var task in tasks)
						task.Start();

					await Task.WhenAll(tasks);
					global.Stop();
					labelControlTime.Text = global.Elapsed.ToString();
					labelControlEndDate.Text = DateTime.Now.ToString();
					labelControlSumTime.Text = sumTime.ToString();
					labelControlMean.Text = new TimeSpan(sumTime.Ticks / (threadNumber * opeartions)).ToString();

					foreach (var session in sessions)
						testRepository.CloseSession(session);
				}
				else
				{
					richTextBoxResults.Text = "Guid sesji\tTreść dokumentu\tCzas\tStatus" + Environment.NewLine;

					var _years = yearsRepository.GetStatic();
					if (!_years.IsSucceed)
					{
						messagesService.AddMessage(_years);
						return;
					}
					yearId = _years.Object.LastOrDefault().Id;

					var _documentTypes = documentsRepository.GetDocumentTypes(yearId);
					if (!_documentTypes.IsSucceed)
					{
						messagesService.AddMessage(_documentTypes);
						return;
					}
					documentType = _documentTypes.Object.Where(x => x.Character == Interface.Enums.enumFKDocumentCharacter.DP).FirstOrDefault().Code;

					var _accountsChart = accountsChartRepository.GetFlatList(yearId);
					if (!_accountsChart.IsSucceed)
					{
						messagesService.AddMessage(_accountsChart);
						return;
					}
					accountCharts = _accountsChart.Object;
					accountCharts.RemoveAll(x => x.Account.ToLower().Contains("x"));

					List<Task> tasks = new List<Task>();
					List<string> sessions = new List<string>();
					for (int i = 0; i < threadNumber; i++)
					{
						var _result = sessionRepository.OpenNewSession($"Test dokumentu prostego {i + 1}");
						if (_result.IsSucceed)
						{
							Task task = new Task(() => createTest(opeartions, _result.Object));
							tasks.Add(task);
							sessions.Add(_result.Object);
						}
						else
						{
							messagesService.AddMessage(_result);
							return;
						}
					}

					sumTime = new TimeSpan();
					Stopwatch global = new Stopwatch();
					labelControlStartDate.Text = DateTime.Now.ToString();

					global.Start();
					foreach (var task in tasks)
						task.Start();

					await Task.WhenAll(tasks);
					global.Stop();
					labelControlTime.Text = global.Elapsed.ToString();
					labelControlEndDate.Text = DateTime.Now.ToString();
					labelControlSumTime.Text = sumTime.ToString();
					labelControlMean.Text = new TimeSpan(sumTime.Ticks / (threadNumber * opeartions)).ToString();

					foreach (var session in sessions)
						testRepository.CloseSession(session);
				}
			}
			catch (Exception exc)
			{
				XtraMessageBox.Show(exc.ToString());
			}
			finally
			{
				setEdit(true);
			}
		}
		private void getTest(int operations, string guidSession)
		{
			try
			{
				int threadId = Thread.CurrentThread.ManagedThreadId;
				Random random = new Random(threadId);
				int maxValue = documents.Count() - 1;

				using (HttpClient httpClient = new HttpClient())
				{
					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					IFKDocumentsController documentsHandler = unityContainer.Resolve<IFKDocumentsController>(
						new ParameterOverride("httpClient", httpClient),
						new ParameterOverride("sessionGuid", guidSession));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					for (int i = 0; i < operations; i++)
					{
						var index = random.Next(0, maxValue);
						int id = documents[index].Id;

						stopwatch.Restart();
						var result = documentsHandler.Get(id, yearId) as ResponseMessageResult;
						var succeed = result.Response.IsSuccessStatusCode;
						stopwatch.Stop();

						richTextBoxResults.Invoke((MethodInvoker)delegate
						{
							richTextBoxResults.Text += string.Format("{0}\t{1}\t{2}\t{3}{4}",
								guidSession,
								id,
								stopwatch.Elapsed,
								succeed,
								Environment.NewLine);

							sumTime = sumTime.Add(stopwatch.Elapsed);
						});

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}
				}
			}
			catch
			{ }
		}
		private void createTest(int operations, string guidSession)
		{
			try
			{
				using (HttpClient httpClient = new HttpClient())
				{
					int threadId = Thread.CurrentThread.ManagedThreadId;
					Random random = new Random(threadId);

					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					IFKDocumentsIssueController documentIssueHandler = unityContainer.Resolve<IFKDocumentsIssueController>(
						new ParameterOverride("httpClient", httpClient),
						new ParameterOverride("sessionGuid", guidSession));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					for (int i = 0; i < operations; i++)
					{
						Guid guid = Guid.NewGuid();
						var stringGuid = guid.ToString().Replace("-", "") + "_" + threadId;

						DocumentIssue document = new DocumentIssue();
						document.YearId = yearId;
						document.Content = stringGuid;
						document.DocumentNumber = stringGuid;
						document.DocumentDate = DateTime.Now;
						document.IssueDate = DateTime.Now;
						document.OperationDate = DateTime.Now;
						document.TypeCode = documentType;
						document.Records = new List<DocumentRecordIssue>();

						int recordNumber = random.Next(1, 5);
						for (int j = 0; j < recordNumber; j++)
						{
							decimal value = random.Next(1, 100);
							for (int k = 0; k <= 1; k++)
							{
								DocumentRecordIssue record = new DocumentRecordIssue();
								record.Side = (enumSideType)k;
								record.No = j;
								record.SplitNo = k;
								record.ParallelType = enumParallelType.Undefined;
								record.DescriptionKind = enumRecordDecsriptionKind.FromDocument;
								record.Description = stringGuid + j;
								record.ValuePLN = value;

								var debitAccount = accountCharts[random.Next(0, accountCharts.Count - 1)];
								record.Account = new DocumentRecordAccountIssue()
								{
									Synthetic = debitAccount.Account_Full.Synthetic,
									Analytic1 = debitAccount.Account_Full.Analytic1,
									Analytic2 = debitAccount.Account_Full.Analytic2,
									Analytic3 = debitAccount.Account_Full.Analytic3,
									Analytic4 = debitAccount.Account_Full.Analytic4,
									Analytic5 = debitAccount.Account_Full.Analytic5,
								};

								document.Records.Add(record);
							}
						}

						stopwatch.Restart();
						var result = documentIssueHandler.AddNew(document) as ResponseMessageResult;
						var succeed = result.Response.IsSuccessStatusCode;
						stopwatch.Stop();

						richTextBoxResults.Invoke((MethodInvoker)delegate
						{
							richTextBoxResults.Text += string.Format("{0}\t{1}\t{2}\t{3}{4}",
								guidSession,
								stringGuid,
								stopwatch.Elapsed,
								succeed,
								Environment.NewLine);

							sumTime = sumTime.Add(stopwatch.Elapsed);
						});

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}
				}
			}
			catch (Exception exc)
			{
			}
		}

		private void initialize()
		{
			List<Tuple<int, string>> types = new List<Tuple<int, string>>();
			types.Add(new Tuple<int, string>(0, "Pobieranie"));
			types.Add(new Tuple<int, string>(1, "Dodawanie"));
			lookUpEditType.Properties.DataSource = types;
			lookUpEditType.Properties.DisplayMember = "Item2";
			lookUpEditType.Properties.ValueMember = "Item1";

			lookUpEditType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));
			lookUpEditType.EditValue = 0;

			spinEditOperationsInThreads.Properties.MaxValue = int.MaxValue;
			spinEditThreads.Properties.MaxValue = int.MaxValue;
		}
		private void setEdit(bool mode)
		{
			lookUpEditType.Enabled = mode;
			spinEditOperationsInThreads.Enabled = mode;
			spinEditThreads.Enabled = mode;
			FormMain?.SetEdit(mode);
			edit = mode;
		}

		public bool CanClose()
		{
			return edit;
		}
	}
}
