﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Unity;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Demo.Model;
using DevExpress.XtraGrid.Columns;
using System.IO;
using DevExpress.XtraEditors;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.FKF.Common.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Documents.ViewModels;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlFKDocumentsIndex : ManagedControl
	{
		public override Guid Identifier => new Guid("4E2B8E79-81F0-4949-814B-0B39E48A7B92");

		private readonly IUnityContainer unityContainer;
		private readonly GridControlService gridControlService;
		private readonly DockManagerService dockManagerService;
		private readonly MessagesService messagesService;
		private readonly FKDocumentsRepository documentsRepository;
		private readonly FKYearsRepository yearsRepository;

		private List<DocumentListElement> documents;

		public ControlFKDocumentsIndex(IUnityContainer unityContainer, GridControlService gridControlService, DockManagerService dockManagerService, MessagesService messagesService,
			FKDocumentsRepository documentsRepository, FKYearsRepository yearsRepository)
		{
			this.unityContainer = unityContainer;
			this.gridControlService = gridControlService;
			this.dockManagerService = dockManagerService;
			this.messagesService = messagesService;
			this.documentsRepository = documentsRepository;
			this.yearsRepository = yearsRepository;
			InitializeComponent();
			Text = "Kartoteka dokumentów w buforze";
		}

		private void ControlForeignOrdersIndex_Load(object sender, EventArgs e)
		{
			try
			{
				barStaticItemGuid.Caption = unityContainer.Resolve<string>("SessionGuid");
				loadStaticData();
				loadData();
				gridViewForeignOrders.PopulateColumns();

				gridControlService.SetColumnReadOnly(ref gridViewForeignOrders, true);
				gridControlService.SetLocalization(ref gridViewForeignOrders, localizations);
				gridViewForeignOrders.Columns[nameof(DocumentListElement.YearId)].Visible = false;

				repositoryItemLookUpEditYear.PopulateColumns();
				repositoryItemLookUpEditYear.Columns[nameof(Year.Id)].Visible = false;
				repositoryItemLookUpEditYear.Columns[nameof(Year.Archived)].Visible = false;
				repositoryItemLookUpEditYear.Columns[nameof(Year.Closed)].Visible = false;
				repositoryItemLookUpEditYear.Columns[nameof(Year.EndDate)].Visible = false;
				repositoryItemLookUpEditYear.Columns[nameof(Year.StartDate)].Visible = false;
				repositoryItemLookUpEditYear.Columns[nameof(Year.Length)].Visible = false;
				repositoryItemLookUpEditYear.Columns[nameof(Year.Code)].Caption = "Kod";
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemNew_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				int yearId = -1;
				DialogResult dr = DialogResult.Cancel;
				using (FormYear frm = unityContainer.Resolve<FormYear>())
				{
					dr = frm.ShowDialog();
					if (dr == DialogResult.OK)
						yearId = frm.YearId;
				}

				if (dr == DialogResult.OK)
				{
					SplashScreenManager.ShowForm(typeof(FormWaiting));

					var control = unityContainer.Resolve<ControlFKDocument>();
					control.YearId = yearId;
					dockManagerService.OpenPanel(control);
				}
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemRefresh_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));
				loadData();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void loadData()
		{
			DateTime? dateFrom = (DateTime?)barEditItemDateFrom.EditValue;
			DateTime? dateTo = (DateTime?)barEditItemDateTo.EditValue;
			int yearId = Convert.ToInt32(barEditItemYear.EditValue);

			var response = documentsRepository.GetFromBuffer(yearId, dateFrom, dateTo);
			if (response.IsSucceed)
			{
				documents = response.Object;
				gridControlForeignOrders.DataSource = documents;
				gridControlForeignOrders.RefreshDataSource();
			}
			else
				messagesService.AddMessage(response);
		}
		private void loadStaticData()
		{
			var _years = yearsRepository.GetStatic();
			if (_years.IsSucceed)
			{
				repositoryItemLookUpEditYear.ValueMember = nameof(Year.Id);
				repositoryItemLookUpEditYear.DisplayMember = nameof(Year.Code);
				repositoryItemLookUpEditYear.DataSource = _years.Object;
				barEditItemYear.EditValue = _years.Object.Min(x => x.Id);
			}
			else
				messagesService.AddMessage(_years);
		}
		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "DocumentNumber", "Numer dokumentu" },
			{ "Source", "Źródło" },
			{ "YearId", "Rok" },
			{ "DocumentDate", "Data dokumentu" },
			{ "PeriodDate", "Data okresu" },
			{ "OperationDate", "Data operacji" },
			{ "IssueDate", "Data wystawienia" },
			{ "ReceiptDate", "Data wpływu" },
			{ "TypeCode", "Typ dokumentu" },
			{ "NumberInSeries", "Numer w serii" },
			{ "Content", "Treść" },
			{ "ContractorPosition", "Pozycja" },
			{ "Value", "Wartość" },
			{ "ValuePLN", "Wartość PLN" },
			{ "Currency", "Waluta" },
		};
	}
}
