﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Demo.Extensions;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using DevExpress.XtraEditors;
using Sage.PL.WebAPI.Demo.Forms;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Interface.FKF.Contractors.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Employees.ViewModels;
using Sage.PL.WebAPI.Demo.DAL;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlFKEmployee : ManagedControl
	{
		public override Guid Identifier => new Guid("04EC137A-04FF-42E5-8BBA-0DBFC1EBBF77");

		private Employee employee;
		private readonly FKEmployeesRepository employeesRepository;
		private readonly MessagesService messagesService;

		public ControlFKEmployee(Employee employee, FKEmployeesRepository employeesRepository, MessagesService messagesService)
		{
			InitializeComponent();
			Text = "Pracownik FK";

			this.employee = employee;
			this.employeesRepository = employeesRepository;
			this.messagesService = messagesService;
			intitializeBinding();
		}

		private void ControlContractor_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemSave_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				ResponseResult<Employee> _employee;
				if (employee.Id > 0)
					_employee = employeesRepository.Update(employee);
				else
					_employee = employeesRepository.AddNew(employee);

				if (_employee.IsSucceed)
				{
					if (employee.Id > 0)
					{
						var _employee2 = employeesRepository.GetById(employee.Id.Value);
						if (_employee2.IsSucceed)
							employee = _employee2.Object;
					}
					else
						employee = _employee.Object;

					refresh();
					XtraMessageBox.Show("Zapisano zmiany");
				}
				else
					messagesService.AddMessage(_employee);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemCancel_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				if (employee.Id > 0)
				{
					var _employee = employeesRepository.GetById(employee.Id.Value);
					if (_employee.IsSucceed)
						employee = _employee.Object;
					else
						messagesService.AddMessage(_employee);
				}
				else
				{
					employee = new Employee()
					{
						Address = new EmployeeAddress()
					};
				}

				refresh();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void initialize()
		{
			if (employee.Id > 0)
			{
				textEditCode.ReadOnly = true;
				textEditPosition.ReadOnly = true;
			}
		}
		private void intitializeBinding()
		{
			textEditCode.DataBindings.Add("EditValue", employee, nameof(employee.Code), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditName.DataBindings.Add("EditValue", employee, nameof(employee.Name), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditSurname.DataBindings.Add("EditValue", employee, nameof(employee.Surname), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditActive.DataBindings.Add("EditValue", employee, nameof(employee.Active), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPosition.DataBindings.Add("EditValue", employee, nameof(employee.Position), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditCity.DataBindings.Add("EditValue", employee, nameof(employee.Address) + "." + nameof(employee.Address.City), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditPostCode.DataBindings.Add("EditValue", employee, nameof(employee.Address) + "." + nameof(employee.Address.PostCode), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditStreet.DataBindings.Add("EditValue", employee, nameof(employee.Address) + "." + nameof(employee.Address.Street), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditHouseNo.DataBindings.Add("EditValue", employee, nameof(employee.Address) + "." + nameof(employee.Address.HouseNo), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditApartmentNo.DataBindings.Add("EditValue", employee, nameof(employee.Address) + "." + nameof(employee.Address.ApartmentNo), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditPesel.DataBindings.Add("EditValue", employee, nameof(employee.Pesel), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditNIP.DataBindings.Add("EditValue", employee, nameof(employee.NIP), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditPhone1.DataBindings.Add("EditValue", employee, nameof(employee.Phone), true, DataSourceUpdateMode.OnPropertyChanged);
		}
		private void refresh()
		{
			textEditCode.DataBindings.Clear();
			textEditName.DataBindings.Clear();
			textEditSurname.DataBindings.Clear();
			checkEditActive.DataBindings.Clear();
			textEditPosition.DataBindings.Clear();

			textEditCity.DataBindings.Clear();
			textEditPostCode.DataBindings.Clear();
			textEditStreet.DataBindings.Clear();
			textEditHouseNo.DataBindings.Clear();
			textEditApartmentNo.DataBindings.Clear();

			textEditPesel.DataBindings.Clear();
			textEditNIP.DataBindings.Clear();

			textEditPhone1.DataBindings.Clear();

			intitializeBinding();
		}
	}
}
