﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Unity;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Interface.Contractors.ViewModels;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Demo.Model;
using DevExpress.XtraGrid.Views.Grid;
using Unity.Resolution;
using DevExpress.XtraEditors;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using Sage.PL.WebAPI.Interface.FKF.Employees.ViewModels;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlFKEmployeesIndex : ManagedControl
	{
		public override Guid Identifier => new Guid("7AAF7E59-F398-4750-9D55-CD2384613E1A");

		private readonly IUnityContainer unityContainer;
		private readonly GridControlService gridControlService;
		private readonly DockManagerService dockManagerService;
		private readonly MessagesService messagesService;
		private readonly FKEmployeesRepository employeesRepository;

		private List<EmployeeListElement> employees;

		public ControlFKEmployeesIndex(IUnityContainer unityContainer, GridControlService gridControlService, DockManagerService dockManagerService, MessagesService messagesService,
			FKEmployeesRepository employeesRepository)
		{
			this.unityContainer = unityContainer;
			this.gridControlService = gridControlService;
			this.dockManagerService = dockManagerService;
			this.messagesService = messagesService;
			this.employeesRepository = employeesRepository;
			InitializeComponent();
			Text = "Kartoteka pracowników FK";
		}

		private void ControlContractorsIndex_Load(object sender, EventArgs e)
		{
			try
			{
				barStaticItemGuid.Caption = unityContainer.Resolve<string>("SessionGuid");
				loadData();
				gridViewEmployees.PopulateColumns();

				gridControlService.SetColumnReadOnly(ref gridViewEmployees, true);
				gridControlService.SetLocalization(ref gridViewEmployees, localizations);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemEdit_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				int handle = gridViewEmployees.FocusedRowHandle;
				if (handle < 0)
					return;

				var indexEmployee = gridViewEmployees.GetRow(handle) as EmployeeListElement;
				var response = employeesRepository.GetById(indexEmployee.Id);
				if (response.IsSucceed)
				{
					var control = unityContainer.Resolve<ControlFKEmployee>(new ParameterOverride("employee", response.Object));
					dockManagerService.OpenPanel(control);
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemNew_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				Employee employee = new Employee()
				{
					Address = new EmployeeAddress()
				};

				var control = unityContainer.Resolve<ControlFKEmployee>(new ParameterOverride("employee", employee));
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemRefresh_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));
				loadData();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void gridViewEmployees_DoubleClick(object sender, EventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				GridView gridView = sender as GridView;
				if (gridView == null)
					return;

				int handle = gridView.FocusedRowHandle;
				if (handle < 0)
					return;

				var indexEmployee = gridView.GetRow(handle) as EmployeeListElement;
				var response = employeesRepository.GetById(indexEmployee.Id);
				if (response.IsSucceed)
				{
					var control = unityContainer.Resolve<ControlFKEmployee>(new ParameterOverride("employee", response.Object));
					dockManagerService.OpenPanel(control);
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void loadData()
		{
			var response = employeesRepository.Get();
			if (response.IsSucceed)
			{
				employees = response.Object;
				gridControlEmployees.DataSource = employees;
				gridControlEmployees.RefreshDataSource();
			}
			else
				messagesService.AddMessage(response);
		}
		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "Code", "Kod" },
			{ "Name", "Nazwa" },
			{ "Position", "Pozycja" },
			{ "Type", "Typ" },
			{ "Active", "Aktywny" },
			{ "Place", "Miejscowość" },
		};
	}
}
