﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using DevExpress.XtraEditors;
using Sage.PL.WebAPI.Demo.ViewModels;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.Common.ViewModels;
using DevExpress.XtraEditors.Repository;
using DevExpress.XtraGrid.Views.BandedGrid;
using DevExpress.XtraGrid.Columns;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Demo.Extensions;
using Sage.PL.WebAPI.Demo.Model;
using Sage.PL.WebAPI.Demo.Mappers;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlForeignOrder : ManagedControl
	{
		public override Guid Identifier => new Guid("3765280C-AC11-461D-97E3-32FE5CD9E310");

		private readonly ForeignOrdersRepository foreignOrdersRepository;
		private readonly ContractorsRepository contractorsRepository;
		private readonly ProductsRepository productsRepository;
		private readonly PaymentRegistriesRepository paymentRegistriesRepository;
		private readonly PaymentFormsRepository paymentFormsRepository;
		private readonly DepartmentsRepository departmentsRepository;
		private readonly PricesRepository pricesRepository;
		private readonly ForeignOrderMapper foreignOrderMapper;
		private readonly GridControlService gridControlService;
		private readonly MessagesService messagesService;
		private OrderViewModel order;

		public ControlForeignOrder(ForeignOrdersRepository foreignOrdersRepository, ContractorsRepository contractorsRepository, ProductsRepository productsRepository,
			PaymentRegistriesRepository paymentRegistriesRepository, PaymentFormsRepository paymentFormsRepository, DepartmentsRepository departmentsRepository,
			PricesRepository pricesRepository, ForeignOrderMapper foreignOrderMapper,
			GridControlService gridControlService, MessagesService messagesService)
		{
			InitializeComponent();
			Text = "Zamówienie obce";

			order = new OrderViewModel()
			{
				Buyer = new Interface.Contractors.ViewModels.ContractorListElement(),
				Recipient = new Interface.Contractors.ViewModels.ContractorListElement(),
				DocumentType = new DocumentType(),
				DocumentSeries = new DocumentSeries(),
				PaymentForm = new PaymentForm(),
				PaymentRegistry = new PaymentRegistry(),
				Department = new Department(),
				IssueDate = DateTime.Now,
				MaturityDate = DateTime.Now,
				SaleDate = DateTime.Now,
				Positions = new List<OrderPositionViewModel>()

			};
			this.foreignOrdersRepository = foreignOrdersRepository;
			this.contractorsRepository = contractorsRepository;
			this.productsRepository = productsRepository;
			this.paymentRegistriesRepository = paymentRegistriesRepository;
			this.paymentFormsRepository = paymentFormsRepository;
			this.departmentsRepository = departmentsRepository;
			this.pricesRepository = pricesRepository;
			this.foreignOrderMapper = foreignOrderMapper;
			this.gridControlService = gridControlService;
			this.messagesService = messagesService;
			intitializeBinding();
		}

		private void ControlForeignOrder_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemSave_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var orderIssue = foreignOrderMapper.Map(order);
				var _order = foreignOrdersRepository.AddNew(orderIssue);
				if (_order.IsSucceed)
					XtraMessageBox.Show("Zapisano nowe zamówienie obce");
				else
					messagesService.AddMessage(_order);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonAddPosition_Click(object sender, EventArgs e)
		{
			try
			{
				order.Positions.Add(new OrderPositionViewModel() { No = order.Positions.Count + 1 });
				gridControlPositions.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonDelete_Click(object sender, EventArgs e)
		{
			try
			{
				var handle = advBandedGridViewPositions.FocusedRowHandle;
				if (handle < 0)
					return;

				OrderPositionViewModel position = advBandedGridViewPositions.GetRow(handle) as OrderPositionViewModel;
				if (position == null)
					return;

				order.Positions.Remove(position);
				int no = 1;
				order.Positions.ForEach(x => x.No = no++);
				gridControlPositions.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}

		private void advBandedGridViewPositions_CellValueChanged(object sender, DevExpress.XtraGrid.Views.Base.CellValueChangedEventArgs e)
		{
			try
			{
				AdvBandedGridView gridView = sender as AdvBandedGridView;
				if (gridView == null)
					return;

				int handle = gridView.FocusedRowHandle;
				if (handle < 0)
					return;

				var position = gridView.GetRow(handle) as OrderPositionViewModel;
				if (position == null)
					return;

				GridColumn column = gridView.FocusedColumn;
				if (column.FieldName.Equals(nameof(OrderPositionViewModel.Qunatity)) || column.FieldName.Equals(nameof(OrderPositionViewModel.Price)))
					gridControlPositions.RefreshDataSource();
			}
			catch (Exception exc)
			{
			}
		}
		private void lookUpEditPaymentRegistry_EditValueChanged(object sender, EventArgs e)
		{
			try
			{
				var registry = lookUpEditPaymentRegistry.EditValue as PaymentRegistry;
				if (registry?.Id != null)
				{
					var paymentForms = paymentFormsRepository.Get(registry.Id.Value);
					if (paymentForms.IsSucceed)
					{
						var payment = lookUpEditPaymentForm.EditValue as PaymentForm;
						if (!paymentForms.Object.Exists(x => x.Id == payment?.Id))
							order.PaymentForm = null;

						lookUpEditPaymentForm.Properties.DataSource = paymentForms.Object;
					}
				}
			}
			catch (Exception exc)
			{
			}
		}
		private void lookUpEditDocumentType_EditValueChanged(object sender, EventArgs e)
		{
			try
			{
				var type = lookUpEditDocumentType.EditValue as DocumentType;
				if (type?.Id != null)
				{
					var _serie = foreignOrdersRepository.GetDocumentSeries(type.Id);
					if (_serie.IsSucceed)
					{
						var serie = lookUpEditDocumentSerie.EditValue as DocumentSeries;
						if (!_serie.Object.Exists(x => x.Id == serie?.Id))
							order.DocumentSeries = null;

						lookUpEditDocumentSerie.Properties.DataSource = _serie.Object;
					}
					else
						messagesService.AddMessage(_serie);
				}
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}

		private void initialize()
		{
			var paymentRegistries = paymentRegistriesRepository.GetStatic();
			if (paymentRegistries.IsSucceed)
			{
				lookUpEditPaymentRegistry.Properties.DataSource = paymentRegistries.Object;
				lookUpEditPaymentRegistry.Properties.DisplayMember = "Name";
				lookUpEditPaymentRegistry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(paymentRegistries);

			lookUpEditPaymentForm.Properties.DisplayMember = "Name";
			lookUpEditPaymentForm.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			lookUpEditPaymentForm.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Days", "Dni"));

			var contractors = contractorsRepository.Get();
			if (contractors.IsSucceed)
			{
				lookUpEditBuyer.Properties.DataSource = contractors.Object;
				lookUpEditBuyer.Properties.DisplayMember = "Code";
				lookUpEditBuyer.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditBuyer.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));

				lookUpEditRecipient.Properties.DataSource = contractors.Object;
				lookUpEditRecipient.Properties.DisplayMember = "Code";
				lookUpEditRecipient.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditRecipient.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(contractors);

			var types = foreignOrdersRepository.GetZMODocumentTypes();
			if (types.IsSucceed)
			{
				lookUpEditDocumentType.Properties.DataSource = types.Object;
				lookUpEditDocumentType.Properties.DisplayMember = "Code";
				lookUpEditDocumentType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditDocumentType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(types);

			lookUpEditDocumentSerie.Properties.DisplayMember = "Code";
			lookUpEditDocumentSerie.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));

			var depart = departmentsRepository.GetStatic();
			if (depart.IsSucceed)
			{
				lookUpEditDepartment.Properties.DataSource = depart.Object;
				lookUpEditDepartment.Properties.DisplayMember = "Code";
				lookUpEditDepartment.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditDepartment.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(depart);

			var _products = productsRepository.Get();
			if (_products.IsSucceed)
			{
				var columnProduct = advBandedGridViewPositions.Columns["Product"];
				gridControlService.AddLookupEdit(ref columnProduct, _products.Object, "", "Code");

				RepositoryItemLookUpEdit ri = columnProduct.ColumnEdit as RepositoryItemLookUpEdit;
				if (ri != null)
				{
					ri.PopulateColumns();
					ri.Columns["Id"].Visible = false;
					ri.Columns["Active"].Visible = false;
					ri.Columns["Unit"].Visible = false;
					ri.Columns["DefaultUnit"].Visible = false;
					ri.Columns["VatRate"].Visible = false;
					ri.Columns["Barcode"].Visible = false;
					ri.Columns["CN"].Visible = false;
					ri.Columns["PKWiU"].Visible = false;
					ri.Columns["FiscalName"].Visible = false;

					ri.Columns["Code"].Caption = "Kod";
					ri.Columns["Name"].Caption = "Nazwa";
					ri.Columns["Type"].Caption = "Typ";

					ri.NullText = string.Empty;
				}
			}
			else
				messagesService.AddMessage(_products);

			List<Tuple<enumPriceKind, string>> priceKinds = new List<Tuple<enumPriceKind, string>>();
			priceKinds.Add(new Tuple<enumPriceKind, string>(enumPriceKind.Net, enumPriceKind.Net.GetAttributeDescription<EnumPolishName>()));
			priceKinds.Add(new Tuple<enumPriceKind, string>(enumPriceKind.Gross, enumPriceKind.Gross.GetAttributeDescription<EnumPolishName>()));

			lookUpEditPriceKind.Properties.DataSource = priceKinds;
			lookUpEditPriceKind.Properties.DisplayMember = "Item2";
			lookUpEditPriceKind.Properties.ValueMember = "Item1";
			lookUpEditPriceKind.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			var _prices = pricesRepository.GetSalePriceTypesStatic();
			if (_prices.IsSucceed)
			{
				List<Tuple<enumSalePriceType, string>> salePrices = new List<Tuple<enumSalePriceType, string>>();
				foreach (var price in _prices.Object.Where(x => x.Active))
					salePrices.Add(new Tuple<enumSalePriceType, string>((enumSalePriceType)price.Type, price.Type.GetAttributeDescription<EnumPolishName>()));

				lookUpEditPriceType.Properties.DataSource = salePrices;
				lookUpEditPriceType.Properties.DisplayMember = "Item2";
				lookUpEditPriceType.Properties.ValueMember = "Item1";
				lookUpEditPriceType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));
			}
			else
				messagesService.AddMessage(_prices);

			gridControlPositions.DataSource = order.Positions;
			gridControlPositions.RefreshDataSource();
		}
		private void intitializeBinding()
		{
			lookUpEditRecipient.DataBindings.Add("EditValue", order, nameof(order.Recipient), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditBuyer.DataBindings.Add("EditValue", order, nameof(order.Buyer), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPaymentRegistry.DataBindings.Add("EditValue", order, nameof(order.PaymentRegistry), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPaymentForm.DataBindings.Add("EditValue", order, nameof(order.PaymentForm), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditBuyer.DataBindings.Add("EditValue", order, nameof(order.BuyerName), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditRecipient.DataBindings.Add("EditValue", order, nameof(order.RecipientName), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditBCity.DataBindings.Add("EditValue", order, nameof(order.Buyer) + "." + nameof(order.Buyer.Place), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditRCity.DataBindings.Add("EditValue", order, nameof(order.Recipient) + "." + nameof(order.Recipient.Place), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditDescription.DataBindings.Add("EditValue", order, nameof(order.Description), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditRecivedBy.DataBindings.Add("EditValue", order, nameof(order.RecivedBy), true, DataSourceUpdateMode.OnPropertyChanged);
			richTextBoxNote.DataBindings.Add("Text", order, nameof(order.Note), true, DataSourceUpdateMode.OnPropertyChanged);

			dateEditIssueDate.DataBindings.Add("EditValue", order, nameof(order.IssueDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditSaleDate.DataBindings.Add("EditValue", order, nameof(order.SaleDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditMaturityDate.DataBindings.Add("EditValue", order, nameof(order.MaturityDate), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditDocumentType.DataBindings.Add("EditValue", order, nameof(order.DocumentType), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditDocumentSerie.DataBindings.Add("EditValue", order, nameof(order.DocumentSeries), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditDepartment.DataBindings.Add("EditValue", order, nameof(order.Department), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditPriceKind.DataBindings.Add("EditValue", order, nameof(order.PriceKind), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPriceType.DataBindings.Add("EditValue", order, nameof(order.SalePriceType), true, DataSourceUpdateMode.OnPropertyChanged);
		}
	}
}
