﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using DevExpress.XtraEditors;
using System.Net.Http;
using Sage.PL.WebAPI.Interface.Contractors.Interfaces;
using System.Threading;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Unity;
using Unity.Resolution;
using System.Diagnostics;
using System.Web.Http.Results;
using Sage.PL.WebAPI.Demo.Forms;
using System.Security.Cryptography;
using Sage.PL.WebAPI.Interface.Orders.ViewModels;
using Sage.PL.WebAPI.Interface.Orders.Interfaces;
using Sage.PL.WebAPI.Interface.Contractors.ViewModels;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.Common.ViewModels;
using Sage.PL.WebAPI.Interface.Products.ViewModels;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlForeignOrderTest : ManagedControl, ICanClose
	{
		private readonly IUnityContainer unityContainer;
		private readonly MessagesService messagesService;
		private readonly HttpClient originalHttpClient;
		private readonly SessionRepository sessionRepository;
		private readonly TestRepository testRepository;
		private readonly ForeignOrdersRepository foreignOrdersRepository;
		private readonly ContractorsRepository contractorsRepository;
		private readonly DepartmentsRepository departmentsRepository;
		private readonly PaymentRegistriesRepository paymentRegistriesRepository;
		private readonly ProductsRepository productsRepository;

		private List<OrderListElement> orders;
		private List<ContractorListElement> contractors;
		private List<Department> deparments;
		private List<PaymentRegistry> registries;
		private List<ProductListElement> products;

		private string orderType = string.Empty;
		private string orderSerie = string.Empty;

		private TimeSpan sumTime;
		private bool edit = true;

		public FormMain FormMain { get; set; }

		public override Guid Identifier => new Guid("4A307AD7-6491-4DE2-AB7D-47230D96BD52");

		public ControlForeignOrderTest(IUnityContainer unityContainer, MessagesService messagesService,
			HttpClient originalHttpClient, SessionRepository sessionRepository, TestRepository testRepository,
			ForeignOrdersRepository foreignOrdersRepository, ContractorsRepository contractorsRepository,
			DepartmentsRepository departmentsRepository, PaymentRegistriesRepository paymentRegistriesRepository, ProductsRepository productsRepository)
		{
			InitializeComponent();
			initialize();
			setEdit(true);
			Text = "Test zamówienia obcego";

			this.unityContainer = unityContainer;
			this.messagesService = messagesService;
			this.originalHttpClient = originalHttpClient;
			this.sessionRepository = sessionRepository;
			this.testRepository = testRepository;
			this.foreignOrdersRepository = foreignOrdersRepository;
			this.contractorsRepository = contractorsRepository;
			this.departmentsRepository = departmentsRepository;
			this.paymentRegistriesRepository = paymentRegistriesRepository;
			this.productsRepository = productsRepository;
			progressBarControl.Properties.Step = 1;
			progressBarControl.Properties.PercentView = true;
			progressBarControl.Properties.Minimum = 0;
		}

		private async void barButtonItemStart_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				int type = Convert.ToInt32(lookUpEditType.EditValue);
				int threadNumber = Convert.ToInt32(spinEditThreads.EditValue);
				int operation = Convert.ToInt32(spinEditOperationsInThreads.EditValue);
				labelControlEndDate.Text = "TBA";
				labelControlTime.Text = "TBA";
				labelControlSumTime.Text = "TBA";
				labelControlMean.Text = "TBA";

				progressBarControl.Position = 0;
				progressBarControl.Properties.Maximum = threadNumber * operation;
				setEdit(false);

				if (type == 0)
				{
					richTextBoxResults.Text = "Guid sesji\tId zamówienia\tCzas\tStatus" + Environment.NewLine;
					var _orders = foreignOrdersRepository.Get();
					if (!_orders.IsSucceed)
					{
						messagesService.AddMessage(_orders);
						return;
					}

					orders = _orders.Object;

					List<Task> tasks = new List<Task>();
					List<string> sessions = new List<string>();
					for (int i = 0; i < threadNumber; i++)
					{
						var _result = sessionRepository.OpenNewSession($"Test zamówienia obcego {i + 1}");
						if (_result.IsSucceed)
						{
							Task task = new Task(() => getTest(operation, _result.Object));
							tasks.Add(task);
							sessions.Add(_result.Object);
						}
						else
						{
							messagesService.AddMessage(_result);
							return;
						}
					}

					sumTime = new TimeSpan();
					Stopwatch global = new Stopwatch();
					labelControlStartDate.Text = DateTime.Now.ToString();

					global.Start();
					foreach (var task in tasks)
						task.Start();

					await Task.WhenAll(tasks);
					global.Stop();
					labelControlTime.Text = global.Elapsed.ToString();
					labelControlEndDate.Text = DateTime.Now.ToString();
					labelControlSumTime.Text = sumTime.ToString();
					labelControlMean.Text = new TimeSpan(sumTime.Ticks / (threadNumber * operation)).ToString();

					foreach (var session in sessions)
						testRepository.CloseSession(session);
				}
				else
				{
					richTextBoxResults.Text = "Guid sesji\tOpis zamówienia\tCzas\tStatus" + Environment.NewLine;

					var _contractors = contractorsRepository.Get();
					if (!_contractors.IsSucceed)
					{
						messagesService.AddMessage(_contractors);
						return;
					}
					contractors = _contractors.Object;
					contractors.RemoveAll(x => !x.Active);

					var _departments = departmentsRepository.Get();
					if (!_departments.IsSucceed)
					{
						messagesService.AddMessage(_departments);
						return;
					}
					deparments = _departments.Object;

					var _products = productsRepository.Get();
					if (!_products.IsSucceed)
					{
						messagesService.AddMessage(_products);
						return;
					}
					products = _products.Object;
					products.RemoveAll(x => x.Type != Interface.Enums.enumProductType.Article);
					products.RemoveAll(x => !x.Active);

					var _registries = paymentRegistriesRepository.Get();
					if (!_registries.IsSucceed)
					{
						messagesService.AddMessage(_registries);
						return;
					}
					registries = _registries.Object;
					registries.RemoveAll(x => !x.DefaultPaymentFormId.HasValue);

					var _types = foreignOrdersRepository.GetZMODocumentTypes();
					if (!_types.IsSucceed)
					{
						messagesService.AddMessage(_types);
						return;
					}
					var ordType = _types.Object.FirstOrDefault(x => x.Active);

					var _series = foreignOrdersRepository.GetDocumentSeries(ordType.Id);
					if (!_series.IsSucceed)
					{
						messagesService.AddMessage(_series);
						return;
					}

					orderType = ordType.Code;
					orderSerie = _series.Object.FirstOrDefault().Code;

					List<Task> tasks = new List<Task>();
					List<string> sessions = new List<string>();
					for (int i = 0; i < threadNumber; i++)
					{
						var _result = sessionRepository.OpenNewSession($"Test zamówienia obcego {i + 1}");
						if (_result.IsSucceed)
						{
							Task task = new Task(() => createTest(operation, _result.Object));
							tasks.Add(task);
							sessions.Add(_result.Object);
						}
						else
						{
							messagesService.AddMessage(_result);
							return;
						}
					}

					sumTime = new TimeSpan();
					Stopwatch global = new Stopwatch();
					labelControlStartDate.Text = DateTime.Now.ToString();

					global.Start();
					foreach (var task in tasks)
						task.Start();

					await Task.WhenAll(tasks);
					global.Stop();
					labelControlTime.Text = global.Elapsed.ToString();
					labelControlEndDate.Text = DateTime.Now.ToString();
					labelControlSumTime.Text = sumTime.ToString();
					labelControlMean.Text = new TimeSpan(sumTime.Ticks / (threadNumber * operation)).ToString();

					foreach (var session in sessions)
						testRepository.CloseSession(session);
				}
			}
			catch (Exception exc)
			{
				XtraMessageBox.Show(exc.ToString());
			}
			finally
			{
				setEdit(true);
			}
		}
		private void getTest(int operations, string guidSession)
		{
			try
			{
				int threadId = Thread.CurrentThread.ManagedThreadId;
				Random random = new Random(threadId);
				int maxValue = orders.Count() - 1;

				using (HttpClient httpClient = new HttpClient())
				{
					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					IOrdersController ordersHandler = unityContainer.Resolve<IOrdersController>(
						new ParameterOverride("httpClient", httpClient),
						new ParameterOverride("sessionGuid", guidSession));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					for (int i = 0; i < operations; i++)
					{
						var index = random.Next(0, maxValue);
						int id = orders[index].Id;

						stopwatch.Restart();
						var result = ordersHandler.Get(id) as ResponseMessageResult;
						var succeed = result.Response.IsSuccessStatusCode;
						stopwatch.Stop();

						richTextBoxResults.Invoke((MethodInvoker)delegate
						{
							richTextBoxResults.Text += string.Format("{0}\t{1}\t{2}\t{3}{4}",
								guidSession,
								id,
								stopwatch.Elapsed,
								succeed,
								Environment.NewLine);

							sumTime = sumTime.Add(stopwatch.Elapsed);
						});

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}
				}
			}
			catch
			{ }
		}
		private void createTest(int operations, string guidSession)
		{
			try
			{
				using (HttpClient httpClient = new HttpClient())
				{
					int threadId = Thread.CurrentThread.ManagedThreadId;
					Random random = new Random(threadId);

					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					IOrdersIssueController ordersHandler = unityContainer.Resolve<IOrdersIssueController>(
						new ParameterOverride("httpClient", httpClient),
						new ParameterOverride("sessionGuid", guidSession));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					for (int i = 0; i < operations; i++)
					{
						OrderIssue order = new OrderIssue();
						Guid guid = Guid.NewGuid();
						var stringGuid = guid.ToString().Replace("-", "") + "_" + threadId;
						var cId = contractors[random.Next(0, contractors.Count - 1)].Id;
						var depart = deparments[random.Next(0, deparments.Count - 1)].Code;
						var pr = registries[random.Next(0, registries.Count - 1)];

						var posNumber = random.Next(1, 10);
						List<ProductListElement> tws = new List<ProductListElement>();
						for (int j = 0; j < posNumber; j++)
							tws.Add(products[random.Next(0, products.Count - 1)]);

						order.Description = stringGuid;
						order.Buyer = new OrderIssueContractor() { Id = cId };
						order.Recipient = new OrderIssueContractor() { Id = cId };
						order.Department = depart;
						order.PaymentRegistry = new PaymentRegistryBase() { Id = pr.Id };
						order.PaymentFormId = pr.DefaultPaymentFormId.Value;
						order.TypeCode = orderType;
						order.Series = orderSerie;
						order.PriceKind = Interface.Enums.enumPriceKind.Gross;
						order.SalePriceType = Interface.Enums.enumSalePriceType.PriceA;
						order.Positions = new List<OrderIssuePosition>();
						foreach (var tw in tws)
						{
							OrderIssuePosition position = new OrderIssuePosition()
							{
								Code = tw.Code,
								Name = tw.Name,
								Quantity = 1,
								Value = 1,
								VatRate = new VatRateBase()
								{
									Code = tw.VatRate
								},
								UnitOfMeasurement = tw.Unit
							};

							order.Positions.Add(position);
						}

						stopwatch.Restart();
						var result = ordersHandler.AddNew(order) as ResponseMessageResult;
						var succeed = result.Response.IsSuccessStatusCode;
						stopwatch.Stop();

						richTextBoxResults.Invoke((MethodInvoker)delegate
						{
							richTextBoxResults.Text += string.Format("{0}\t{1}\t{2}\t{3}{4}",
								guidSession,
								stringGuid,
								stopwatch.Elapsed,
								succeed,
								Environment.NewLine);

							sumTime = sumTime.Add(stopwatch.Elapsed);
						});

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}
				}
			}
			catch
			{ }
		}

		private void initialize()
		{
			List<Tuple<int, string>> types = new List<Tuple<int, string>>();
			types.Add(new Tuple<int, string>(0, "Pobieranie"));
			types.Add(new Tuple<int, string>(1, "Dodawanie"));
			lookUpEditType.Properties.DataSource = types;
			lookUpEditType.Properties.DisplayMember = "Item2";
			lookUpEditType.Properties.ValueMember = "Item1";

			lookUpEditType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));
			lookUpEditType.EditValue = 0;

			spinEditOperationsInThreads.Properties.MaxValue = int.MaxValue;
			spinEditThreads.Properties.MaxValue = int.MaxValue;
		}
		private void setEdit(bool mode)
		{
			lookUpEditType.Enabled = mode;
			spinEditOperationsInThreads.Enabled = mode;
			spinEditThreads.Enabled = mode;
			FormMain?.SetEdit(mode);
			edit = mode;
		}

		public bool CanClose()
		{
			return edit;
		}
	}
}
