﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Unity;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Interface.Orders.ViewModels;
using Sage.PL.WebAPI.Demo.Model;
using DevExpress.XtraGrid.Columns;
using System.IO;
using DevExpress.XtraEditors;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlForeignOrdersIndex : ManagedControl
	{
		public override Guid Identifier => new Guid("3641A899-4A28-4E5A-88F1-5577430A936A");

		private readonly IUnityContainer unityContainer;
		private readonly GridControlService gridControlService;
		private readonly DockManagerService dockManagerService;
		private readonly MessagesService messagesService;
		private readonly ForeignOrdersRepository foreignOrderRepository;
		private readonly ForeignOrderMapper foreignOrderMapper;
		private readonly ContractorsRepository contractorsRepository;
		private readonly ContractorMapper contractorMapper;

		private List<ForeignOrderModel> foreignOrders;
		private List<ContractorModel> contractors;

		public ControlForeignOrdersIndex(IUnityContainer unityContainer, GridControlService gridControlService, DockManagerService dockManagerService, MessagesService messagesService,
			ForeignOrdersRepository foreignOrderRepository, ForeignOrderMapper foreignOrderMapper,
			ContractorsRepository contractorsRepository, ContractorMapper contractorMapper)
		{
			this.unityContainer = unityContainer;
			this.gridControlService = gridControlService;
			this.dockManagerService = dockManagerService;
			this.messagesService = messagesService;
			this.foreignOrderRepository = foreignOrderRepository;
			this.foreignOrderMapper = foreignOrderMapper;
			this.contractorsRepository = contractorsRepository;
			this.contractorMapper = contractorMapper;
			InitializeComponent();
			Text = "Kartoteka zamówień obcych";
		}

		private void ControlForeignOrdersIndex_Load(object sender, EventArgs e)
		{
			try
			{
				barStaticItemGuid.Caption = unityContainer.Resolve<string>("SessionGuid");
				loadStaticData();
				loadData();
				gridViewForeignOrders.PopulateColumns();

				gridControlService.SetColumnReadOnly(ref gridViewForeignOrders, true);
				gridControlService.SetLocalization(ref gridViewForeignOrders, localizations);

				GridColumn column;
				column = gridViewForeignOrders.Columns[nameof(ForeignOrderModel.BuyerId)];
				if (column != null)
					gridControlService.AddLookupEdit(ref column, contractors);
				column = gridViewForeignOrders.Columns[nameof(ForeignOrderModel.RecipientId)];
				if (column != null)
					gridControlService.AddLookupEdit(ref column, contractors);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemNew_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlForeignOrder>();
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemPDF_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var handle = gridViewForeignOrders.FocusedRowHandle;
				if (handle < 0)
					return;

				ForeignOrderModel order = gridViewForeignOrders.GetRow(handle) as ForeignOrderModel;
				if (order == null)
					return;

				var message = foreignOrderRepository.GetPDF(order.Id);
				if (message.IsSucceed)
				{
					var pdf = message.Object;
					using (SaveFileDialog sfd = new SaveFileDialog())
					{
						sfd.Filter = "pliki PDF (*.pdf)|*.pdf";
						sfd.RestoreDirectory = true;
						sfd.FileName = order.DocumentNumber.Replace(@"/", "_");

						if (sfd.ShowDialog() == DialogResult.OK)
						{
							var pdfBytes = Convert.FromBase64String(pdf.ContentFile_Base64);
							File.WriteAllBytes(sfd.FileName, pdfBytes);
						}
					}
				}
				else
					messagesService.AddMessage(message);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemRefresh_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				loadData();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void loadData()
		{
			DateTime? dateFrom = (DateTime?)barEditItemDateFrom.EditValue;
			DateTime? dateTo = (DateTime?)barEditItemDateTo.EditValue;

			var message = foreignOrderRepository.GetList(dateFrom, dateTo);
			if (message.IsSucceed)
			{
				var _foreignOrders = message.Object;
				foreignOrders = foreignOrderMapper.Map(_foreignOrders);
				gridControlForeignOrders.DataSource = foreignOrders;
				gridControlForeignOrders.RefreshDataSource();
			}
			else
				messagesService.AddMessage(message);
		}
		private void loadStaticData()
		{
			var message = contractorsRepository.Get();
			if (message.IsSucceed)
			{
				var _contractors = message.Object;
				contractors = contractorMapper.Map(_contractors);
			}
			else
				messagesService.AddMessage(message);
		}
		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "DocumentNumber", "Numer dokumentu" },
			{ "Active", "Aktywny" },
			{ "Buffer", "Bufor" },
			{ "Settled", "Rozliczony" },
			{ "IssueDate", "Data wystawienia" },
			{ "SaleDate", "Data sprzedaży" },
			{ "MaturityDate", "Termin płatności" },
			{ "BuyerId", "Nabywca" },
			{ "RecipientId", "Odbiorca" },
			{ "NetValuePLN", "Wartość netto PLN" },
			{ "VatValuePLN", "Wartość VAT PLN" },
			{ "NetValue", "Wartość netto" },
			{ "GrossValue", "Wartość brutto" },
			{ "Currency", "Waluta" },
		};
	}
}
