﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Sage.PL.WebAPI.Demo.Services;
using System.Net.Http;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Unity;
using Sage.PL.WebAPI.Demo.Forms;
using Sage.PL.WebAPI.Demo.Model;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Demo.Extensions;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.Common.ViewModels;
using Sage.PL.WebAPI.Demo.Mappers;
using DevExpress.XtraGrid.Views.Grid;
using Sage.PL.WebAPI.Interface.Products.ViewModels.PriceCalculations;
using Sage.PL.WebAPI.Interface.Products.Interfaces;
using Unity.Resolution;
using System.Diagnostics;
using System.Web.Http.Results;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.ViewModels;
using DevExpress.Utils;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlPriceTest : ManagedControl, ICanClose
	{
		private readonly IUnityContainer unityContainer;
		private readonly MessagesService messagesService;
		private readonly HttpClient originalHttpClient;
		private readonly SessionRepository sessionRepository;
		private readonly TestRepository testRepository;
		private readonly ContractorsRepository contractorsRepository;
		private readonly ProductsRepository productsRepository;
		private readonly CurrenciesRepository currenciesRepository;
		private readonly DepartmentsRepository departmentsRepository;
		private readonly ForeignOrdersRepository foreignOrdersRepository;
		private readonly DocumentTypeMapper documentTypeMapper;
		private readonly ListElementMapper listElementMapper;
		private List<DocumentTypeModel> documentTypes;
		private List<ListElementModel> contractors;
		private List<ListElementExtModel> products;

		private List<Tuple<PriceOrderResult, TimeSpan>> priceOrders;
		private List<Tuple<PriceCalculationResult, TimeSpan>> priceCalculations;

		private bool edit = true;

		public FormMain FormMain { get; set; }

		public override Guid Identifier => new Guid("B49BB400-EAD0-4D09-A4D0-0E3CAA4F41C0");

		public ControlPriceTest(IUnityContainer unityContainer, MessagesService messagesService,
			HttpClient originalHttpClient, SessionRepository sessionRepository, TestRepository testRepository,
			ContractorsRepository contractorsRepository, ProductsRepository productsRepository, CurrenciesRepository currenciesRepository, DepartmentsRepository departmentsRepository,
			ForeignOrdersRepository foreignOrdersRepository, DocumentTypeMapper documentTypeMapper, ListElementMapper listElementMapper)
		{
			this.unityContainer = unityContainer;
			this.messagesService = messagesService;
			this.originalHttpClient = originalHttpClient;
			this.sessionRepository = sessionRepository;
			this.testRepository = testRepository;
			this.contractorsRepository = contractorsRepository;
			this.productsRepository = productsRepository;
			this.currenciesRepository = currenciesRepository;
			this.departmentsRepository = departmentsRepository;
			this.foreignOrdersRepository = foreignOrdersRepository;
			this.documentTypeMapper = documentTypeMapper;
			this.listElementMapper = listElementMapper;

			InitializeComponent();
			initialize();
			setEdit(true);
			Text = "Test kalkulacji cen";

			progressBarControl.Properties.Step = 1;
			progressBarControl.Properties.PercentView = true;
			progressBarControl.Properties.Minimum = 0;
		}

		private void lookUpEditDocumentType_EditValueChanged(object sender, EventArgs e)
		{
			try
			{
				var type = lookUpEditDocumentType.EditValue as DocumentTypeModel;
				if (type?.Id != null)
				{
					if (!type.Currency)
					{
						lookUpEditCurrency.EditValue = null;
						lookUpEditCurrency.Enabled = false;
					}
					else
						lookUpEditCurrency.Enabled = true;


					var _serie = foreignOrdersRepository.GetDocumentSeries(type.Id);
					if (_serie.IsSucceed)
					{
						var serie = lookUpEditDocumentSerie.EditValue as DocumentSeries;
						lookUpEditDocumentSerie.Properties.DataSource = _serie.Object;
					}
					else
						messagesService.AddMessage(_serie);
				}
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private async void barButtonItemStart_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				gridViewContractors.PostEditor();
				gridViewProducts.PostEditor();

				bool valid = true;

				var department = lookUpEditDepartment.EditValue as Department;
				var documentType = lookUpEditDocumentType.EditValue as DocumentTypeModel;
				var documentSerie = lookUpEditDocumentSerie.EditValue as DocumentSeries;
				var currency = (string)lookUpEditCurrency.EditValue;
				var date = dateEditPriceDate.DateTime;
				var price = (enumSalePriceType?)lookUpEditPriceType.EditValue;

				if (department == null)
				{
					messagesService.AddMessage("Nie wybrano działu firmy");
					valid = false;
				}
				if (!price.HasValue)
				{
					messagesService.AddMessage("Nie wybrano typu ceny");
					valid = false;
				}
				if (documentType == null)
				{
					messagesService.AddMessage("Nie wybrano typu dokumentu zamówienia obcego");
					valid = false;
				}
				if (documentSerie == null)
				{
					messagesService.AddMessage("Nie wybrano serii dokumentu zamówienia obcego");
					valid = false;
				}
				if ((documentType?.Currency ?? false) && string.IsNullOrEmpty(currency))
				{
					messagesService.AddMessage("Nie wybrano waluty");
					valid = false;
				}

				var _sContractors = contractors.Where(x => x.Check);
				var _sProducts = products.Where(x => x.Check);

				if (!_sContractors.Any())
				{
					messagesService.AddMessage("Nie wybrano żadnego kontrahenta");
					valid = false;
				}
				if (!_sProducts.Any())
				{
					messagesService.AddMessage("Nie wybrano żadnego towaru");
					valid = false;
				}

				if (!valid)
					return;

				progressBarControl.Position = 0;
				progressBarControl.Properties.Maximum = _sContractors.Count() * 2;
				setEdit(false);

				List<PriceCalculationCriteria> calcCriteria = new List<PriceCalculationCriteria>();
				foreach (var _contractor in _sContractors)
				{
					PriceCalculationCriteria criteria = new PriceCalculationCriteria();
					criteria.CalculationMethod = enumPriceCalculationMethod.Handel;
					criteria.PriceType = price.Value;
					criteria.DepartmentId = department.Id;
					criteria.Date = date;
					criteria.ContractorId = _contractor.Id;
					criteria.Currency = currency;
					criteria.Products = new List<PriceCalculationProductCriteria>();
					foreach (var _product in _sProducts)
					{
						PriceCalculationProductCriteria product = new PriceCalculationProductCriteria();
						product.Id = _product.Id;
						product.Quantity = _product.Quantity <= decimal.Zero ? 1m : _product.Quantity;
						criteria.Products.Add(product);
					}

					calcCriteria.Add(criteria);
				}

				List<PriceOrderCriteria> orderCriteria = new List<PriceOrderCriteria>();
				foreach (var _contractor in _sContractors)
				{
					PriceOrderCriteria criteria = new PriceOrderCriteria();
					criteria.SalePriceType = price.Value;
					criteria.Department = department.Code;
					criteria.TypeCode = documentType.Code;
					criteria.Series = documentSerie.Code;
					criteria.Date = date;
					criteria.Currency = currency;
					criteria.Contractor = new PriceOrderContractorCriteria() { Id = _contractor.Id, Code = _contractor.Code };
					criteria.Products = new List<PriceOrderProductCriteria>();
					foreach (var _product in _sProducts)
					{
						PriceOrderProductCriteria product = new PriceOrderProductCriteria();
						product.Code = _product.Code;
						product.Quantity = _product.Quantity <= decimal.Zero ? 1m : _product.Quantity;
						criteria.Products.Add(product);
					}

					orderCriteria.Add(criteria);
				}

				List<Task> tasks = new List<Task>();
				List<string> sessions = new List<string>();

				var _result1 = sessionRepository.OpenNewSession($"Wyliczanie cen");
				if (_result1.IsSucceed)
				{
					Task task = new Task(() => testCalculate(calcCriteria, _result1.Object));
					tasks.Add(task);
					sessions.Add(_result1.Object);
				}
				else
				{
					messagesService.AddMessage(_result1);
					return;
				}

				var _result2 = sessionRepository.OpenNewSession($"Zamówienia cen");
				if (_result2.IsSucceed)
				{
					Task task = new Task(() => testOrder(orderCriteria, _result2.Object));
					tasks.Add(task);
					sessions.Add(_result2.Object);
				}
				else
				{
					messagesService.AddMessage(_result2);
					return;
				}

				foreach (var task in tasks)
				{
					task.Start();
					await Task.WhenAll(task);
				}

				foreach (var session in sessions)
					testRepository.CloseSession(session);

				var rounding = (int)lookUpEditRounding.EditValue;
				List<PriceResultViewModel> results = new List<PriceResultViewModel>();
				foreach (var _contractor in _sContractors)
				{
					var _erpResults = priceOrders?.FirstOrDefault(x => x.Item1.Contractor.Id == _contractor.Id)?.Item1;
					var _wahResults = priceCalculations?.FirstOrDefault(x => x.Item1.ContractorId == _contractor.Id)?.Item1;

					foreach (var _product in _sProducts)
					{
						PriceResultViewModel result = new PriceResultViewModel();
						result.ProductId = _product.Id;
						result.ProductCode = _product.Code;
						result.ProductName = _product.Name;
						result.PriceERP_Raw = _erpResults?.Products?.FirstOrDefault(x => x.Id == _product.Id)?.Price;
						result.PriceWAH_Raw = _wahResults?.Products?.FirstOrDefault(x => x.Id == _product.Id)?.Price;

						if (result.PriceERP_Raw.HasValue)
							result.PriceERP = decimal.Round(result.PriceERP_Raw.Value, rounding, MidpointRounding.AwayFromZero);
						if (result.PriceWAH_Raw.HasValue)
							result.PriceWAH = decimal.Round(result.PriceWAH_Raw.Value, rounding, MidpointRounding.AwayFromZero);
						result.Difference = result.PriceERP - result.PriceWAH;

						result.ContractorId = _contractor.Id;
						result.ContractorCode = _contractor.Code;
						result.ContractorName = _contractor.Name;
						result.TimeERP = priceOrders?.FirstOrDefault(x => x.Item1.Contractor.Id == _contractor.Id)?.Item2;
						result.TimeWAH = priceCalculations?.FirstOrDefault(x => x.Item1.ContractorId == _contractor.Id)?.Item2;

						results.Add(result);
					}
				}

				gridControlResults.DataSource = results;
				gridControlResults.RefreshDataSource();
				gridViewResults.ExpandAllGroups();

				gridViewResults.Columns[nameof(PriceResultViewModel.PriceERP)].DisplayFormat.FormatType = FormatType.Numeric;
				gridViewResults.Columns[nameof(PriceResultViewModel.PriceERP)].DisplayFormat.FormatString = "n" + rounding;
				gridViewResults.Columns[nameof(PriceResultViewModel.PriceWAH)].DisplayFormat.FormatType = FormatType.Numeric;
				gridViewResults.Columns[nameof(PriceResultViewModel.PriceWAH)].DisplayFormat.FormatString = "n" + rounding;
				gridViewResults.Columns[nameof(PriceResultViewModel.Difference)].DisplayFormat.FormatType = FormatType.Numeric;
				gridViewResults.Columns[nameof(PriceResultViewModel.Difference)].DisplayFormat.FormatString = "n" + rounding;
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
				setEdit(true);
			}
		}

		private void initialize()
		{
			dateEditPriceDate.EditValue = DateTime.Now;
			dateEditPriceDate.Properties.MaxValue = DateTime.Now;

			List<PriceTypeModel> ptModels = new List<PriceTypeModel>();
			foreach (enumSalePriceType pt in (enumSalePriceType[])Enum.GetValues(typeof(enumSalePriceType)))
				ptModels.Add(new PriceTypeModel() { Type = pt, Code = pt.GetAttributeDescription<EnumPolishName>() });
			ptModels.RemoveAt(0);

			lookUpEditPriceType.Properties.DataSource = ptModels;
			lookUpEditPriceType.Properties.DisplayMember = nameof(PriceTypeModel.Code);
			lookUpEditPriceType.Properties.ValueMember = nameof(PriceTypeModel.Type);
			lookUpEditPriceType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo(nameof(PriceTypeModel.Code), "Typ"));

			List<RoundingModel> rModels = new List<RoundingModel>();
			foreach (enumRounding r in (enumRounding[])Enum.GetValues(typeof(enumRounding)))
				rModels.Add(new RoundingModel() { Rounding = r, Code = r.GetDescription() });

			lookUpEditRounding.Properties.DataSource = rModels;
			lookUpEditRounding.Properties.DisplayMember = nameof(RoundingModel.Code);
			lookUpEditRounding.Properties.ValueMember = nameof(RoundingModel.Rounding);
			lookUpEditRounding.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo(nameof(RoundingModel.Code), "Zaokrąglenie"));
			lookUpEditRounding.EditValue = enumRounding._0_01;

			var _currencies = currenciesRepository.GetStatic();
			if (_currencies.IsSucceed)
			{
				lookUpEditCurrency.Properties.DataSource = _currencies.Object;
				lookUpEditCurrency.Properties.DisplayMember = nameof(Currency.Shortcut);
				lookUpEditCurrency.Properties.ValueMember = nameof(Currency.Shortcut);
				lookUpEditCurrency.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo(nameof(Currency.Shortcut), "Waluta"));
				lookUpEditCurrency.Enabled = false;
			}
			else
				messagesService.AddMessage(_currencies);

			var _deparments = departmentsRepository.GetStatic();
			if (_deparments.IsSucceed)
			{
				lookUpEditDepartment.Properties.DataSource = _deparments.Object;
				lookUpEditDepartment.Properties.DisplayMember = nameof(Department.Code);
				lookUpEditDepartment.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo(nameof(Department.Code), "Kod"));
			}
			else
				messagesService.AddMessage(_deparments);

			documentTypes = new List<DocumentTypeModel>();
			var _zmoTypes = foreignOrdersRepository.GetZMODocumentTypes();
			if (_zmoTypes.IsSucceed)
				documentTypes.AddRange(documentTypeMapper.Map(_zmoTypes.Object, false));
			else
				messagesService.AddMessage(_zmoTypes);

			var _zwoTypes = foreignOrdersRepository.GetZWODocumentTypes();
			if (_zwoTypes.IsSucceed)
				documentTypes.AddRange(documentTypeMapper.Map(_zwoTypes.Object, true));
			else
				messagesService.AddMessage(_zwoTypes);

			lookUpEditDocumentType.Properties.DataSource = documentTypes;
			lookUpEditDocumentType.Properties.DisplayMember = nameof(DocumentTypeModel.Code);
			lookUpEditDocumentType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo(nameof(DocumentTypeModel.Code), "Kod"));

			lookUpEditDocumentSerie.Properties.DisplayMember = "Code";
			lookUpEditDocumentSerie.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));


			var _products = productsRepository.Get();
			if (_products.IsSucceed)
			{
				products = listElementMapper.Map(_products.Object.Where(x => x.Active));
				gridControlProducts.DataSource = products;
				gridControlProducts.RefreshDataSource();
				gridViewProducts.PopulateColumns();

			}
			else
				messagesService.AddMessage(_products);

			var _contractors = contractorsRepository.Get();
			if (_contractors.IsSucceed)
			{
				contractors = listElementMapper.Map(_contractors.Object.Where(x => x.Active));
				gridControlContractors.DataSource = contractors;
				gridControlContractors.RefreshDataSource();
				gridViewContractors.PopulateColumns();

			}
			else
				messagesService.AddMessage(_contractors);

			var tmp = new List<PriceResultViewModel>();
			gridControlResults.DataSource = tmp;
			gridControlResults.RefreshDataSource();
			gridViewResults.PopulateColumns();

			gridViewResults.Columns[nameof(PriceResultViewModel.GroupText)].GroupIndex = 1;
		}
		private void setEdit(bool mode)
		{
			lookUpEditDepartment.Enabled = mode;
			lookUpEditDocumentSerie.Enabled = mode;
			lookUpEditDocumentType.Enabled = mode;
			lookUpEditPriceType.Enabled = mode;
			lookUpEditRounding.Enabled = mode;

			if (lookUpEditDocumentType.EditValue is DocumentTypeModel dt && dt.Currency)
				lookUpEditCurrency.Enabled = mode;
			else
				lookUpEditCurrency.Enabled = false;

			dateEditPriceDate.Enabled = mode;
			gridControlContractors.Enabled = mode;
			gridControlProducts.Enabled = mode;
			gridControlResults.Enabled = mode;

			FormMain?.SetEdit(mode);
			edit = mode;
		}

		public bool CanClose()
		{
			return edit;
		}

		private void gridView_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			try
			{
				if (!edit)
					return;

				GridView view = sender as GridView;
				if (e.Button == System.Windows.Forms.MouseButtons.Right)
					DoShowMenu(view.CalcHitInfo(new Point(e.X, e.Y)));
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void DoShowMenu(DevExpress.XtraGrid.Views.Grid.ViewInfo.GridHitInfo hi)
		{
			if (hi.HitTest == DevExpress.XtraGrid.Views.Grid.ViewInfo.GridHitTest.Row ||
				hi.HitTest == DevExpress.XtraGrid.Views.Grid.ViewInfo.GridHitTest.RowCell ||
				hi.HitTest == DevExpress.XtraGrid.Views.Grid.ViewInfo.GridHitTest.RowEdge)
			{
				GridMenu menu = null;
				if (hi.View == gridViewContractors)
					menu = new GridMenu(hi.View);
				else
					menu = new GridMenuRandom(hi.View);

				menu.Init(hi);
				menu.Show(hi.HitPoint);
			}
		}

		private void testCalculate(List<PriceCalculationCriteria> calcCriteria, string guidSession)
		{
			try
			{
				priceCalculations = new List<Tuple<PriceCalculationResult, TimeSpan>>();
				using (HttpClient httpClient = new HttpClient())
				{
					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					ProductPricesRepository productPrices = unityContainer.Resolve<ProductPricesRepository>(
						new ParameterOverride("productPricesController", unityContainer.Resolve<IProductPricesController>(
							new ParameterOverride("httpClient", httpClient),
							new ParameterOverride("sessionGuid", guidSession))));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					foreach (var criteria in calcCriteria)
					{
						stopwatch.Restart();
						var result = productPrices.CalculatePrices(criteria);
						var succeed = result.IsSucceed;
						stopwatch.Stop();

						if (succeed)
							priceCalculations.Add(Tuple.Create(result.Object, stopwatch.Elapsed));
						else
							messagesService.AddMessage(result);

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}


				}
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}

		}
		private void testOrder(List<PriceOrderCriteria> orderCriteria, string guidSession)
		{
			try
			{
				priceOrders = new List<Tuple<PriceOrderResult, TimeSpan>>();
				using (HttpClient httpClient = new HttpClient())
				{
					httpClient.BaseAddress = originalHttpClient.BaseAddress;
					ProductPricesRepository productPrices = unityContainer.Resolve<ProductPricesRepository>(
						new ParameterOverride("productPricesController", unityContainer.Resolve<IProductPricesController>(
							new ParameterOverride("httpClient", httpClient),
							new ParameterOverride("sessionGuid", guidSession))));

					Stopwatch stopwatch = new Stopwatch();
					stopwatch.Start();
					foreach (var criteria in orderCriteria)
					{
						stopwatch.Restart();
						var result = productPrices.OrderPrices(criteria);
						var succeed = result.IsSucceed;
						stopwatch.Stop();

						if (succeed)
							priceOrders.Add(Tuple.Create(result.Object, stopwatch.Elapsed));
						else
							messagesService.AddMessage(result);

						progressBarControl.Invoke((MethodInvoker)delegate
						{
							progressBarControl.PerformStep();
							progressBarControl.Update();
						});
					}

				}
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
		}
	}
}
