﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Sage.PL.WebAPI.Interface.Products.ViewModels;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Demo.Extensions;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Columns;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Demo.Model;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.Common.ViewModels;
using DevExpress.XtraEditors.Repository;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using Sage.PL.WebAPI.Demo.DAL;
using Sage.PL.WebAPI.Interface.Products.ViewModels.Prices;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlProduct : ManagedControl
	{
		public override Guid Identifier => new Guid("58AB547F-241A-4BC9-B7C9-E1145CA2B303");

		private Product product;
		private readonly GridControlService gridControlService;
		private readonly MessagesService messagesService;
		private readonly PricesRepository pricesRepository;
		private readonly ProductsRepository productsRepository;
		private readonly CurrenciesRepository currenciesRepository;
		private readonly UnitsOfMeasurementRepository unitsOfMeasurementRepository;

		public ControlProduct(Product product, GridControlService gridControlService, MessagesService messagesService, PricesRepository pricesRepository,
			ProductsRepository productsRepository, CurrenciesRepository currenciesRepository, UnitsOfMeasurementRepository unitsOfMeasurementRepository)
		{
			InitializeComponent();
			Text = "Towar";

			this.product = product;
			this.gridControlService = gridControlService;
			this.messagesService = messagesService;
			this.pricesRepository = pricesRepository;
			this.productsRepository = productsRepository;
			this.currenciesRepository = currenciesRepository;
			this.unitsOfMeasurementRepository = unitsOfMeasurementRepository;
			intitializeBinding();
		}

		private void ControlProduct_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemSave_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				gridViewSalePrices.PostEditor();

				product.SalePriceRecalculate = true;
				if (product.VatRate == null)
					product.VatRate = new VatRateBase() { Code = "23%" };

				ResponseResult<Product> _product;
				if (product.Id > 0)
					_product = productsRepository.Update(product);
				else
					_product = productsRepository.AddNew(product);

				if (_product.IsSucceed)
				{
					if (product.Id > 0)
					{
						var _contractors2 = productsRepository.Get(product.Id.Value);
						if (_contractors2.IsSucceed)
							product = _contractors2.Object;
					}
					else
						product = _product.Object;
					product.SalePrices.Remove(null);

					refresh();
					XtraMessageBox.Show("Zapisano zmiany");
				}
				else
					messagesService.AddMessage(_product);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemCancel_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				if (product.Id > 0)
				{
					var _product = productsRepository.Get(product.Id.Value);
					if (_product.IsSucceed)
						product = _product.Object;
					else
						messagesService.AddMessage(_product);
				}
				else
				{
					var response = pricesRepository.GetSalePriceTypesStatic();
					if (response.IsSucceed)
					{
						product = new Product()
						{
							UnitsOfMeasurement = new ProductUnitsOfMeasurement(),
							PurchasePrice = new ProductPrice(),
							BasePrice = new ProductBasePrice(),
							SalePrices = new List<ProductSalePrice>()
						};

						foreach (var price in response.Object.Where(x => x.Active))
							product.SalePrices.Add(new ProductSalePrice() { Type = (enumSalePriceType)price.Type, Kind = price.Kind });
					}
					else
						messagesService.AddMessage(response);
				}

				product.SalePrices.Remove(null);
				refresh();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void gridViewSalePrices_ShowingEditor(object sender, CancelEventArgs e)
		{
			try
			{
				GridView gridView = sender as GridView;
				if (gridView == null)
					return;

				int handle = gridView.FocusedRowHandle;
				if (handle < 0)
					return;

				var product = gridView.GetRow(handle) as ProductSalePrice;
				if (product == null)
					return;

				GridColumn column = gridView.FocusedColumn;
				switch (product.PriceParameter)
				{
					case enumPriceParameter.Profit:
						if (column.FieldName.Equals(nameof(ProductSalePrice.MarkupPercent)))
							e.Cancel = true;
						break;
					case enumPriceParameter.Markup:
						if (column.FieldName.Equals(nameof(ProductSalePrice.ProfitPercent)))
							e.Cancel = true;
						break;
				}

				if (product.AutomaticValue && column.FieldName.Equals(nameof(ProductSalePrice.Value)))
					e.Cancel = true;
			}
			catch (Exception exc)
			{
			}
		}

		private void initialize()
		{
			List<Tuple<enumSettlementMethod, string>> settlementMethods = new List<Tuple<enumSettlementMethod, string>>();
			settlementMethods.Add(new Tuple<enumSettlementMethod, string>(enumSettlementMethod.FIFO, enumSettlementMethod.FIFO.GetAttributeDescription<EnumPolishName>()));
			settlementMethods.Add(new Tuple<enumSettlementMethod, string>(enumSettlementMethod.LIFO, enumSettlementMethod.LIFO.GetAttributeDescription<EnumPolishName>()));

			lookUpEditSettlementMethod.Properties.DataSource = settlementMethods;
			lookUpEditSettlementMethod.Properties.DisplayMember = "Item2";
			lookUpEditSettlementMethod.Properties.ValueMember = "Item1";

			lookUpEditSettlementMethod.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			List<Tuple<enumPriceRecalculateType, string>> recalculateTypes = new List<Tuple<enumPriceRecalculateType, string>>();
			recalculateTypes.Add(new Tuple<enumPriceRecalculateType, string>(enumPriceRecalculateType.PurchasePriceAndBasePriceAreNotAgreed, enumPriceRecalculateType.PurchasePriceAndBasePriceAreNotAgreed.GetAttributeDescription<EnumPolishName>()));
			recalculateTypes.Add(new Tuple<enumPriceRecalculateType, string>(enumPriceRecalculateType.AutomaticallyAtEveryPurchasePriceChange, enumPriceRecalculateType.AutomaticallyAtEveryPurchasePriceChange.GetAttributeDescription<EnumPolishName>()));
			recalculateTypes.Add(new Tuple<enumPriceRecalculateType, string>(enumPriceRecalculateType.AutomaticallyWhenPurchasePriceIsHigherThanBasePrice, enumPriceRecalculateType.AutomaticallyWhenPurchasePriceIsHigherThanBasePrice.GetAttributeDescription<EnumPolishName>()));
			recalculateTypes.Add(new Tuple<enumPriceRecalculateType, string>(enumPriceRecalculateType.ManuallyAtEveryPurchasePriceChange, enumPriceRecalculateType.ManuallyAtEveryPurchasePriceChange.GetAttributeDescription<EnumPolishName>()));
			recalculateTypes.Add(new Tuple<enumPriceRecalculateType, string>(enumPriceRecalculateType.ManuallyWhenPurchasePriceIsHigherThanBasePrice, enumPriceRecalculateType.ManuallyWhenPurchasePriceIsHigherThanBasePrice.GetAttributeDescription<EnumPolishName>()));

			lookUpEditRecalculateMethod.Properties.DataSource = recalculateTypes;
			lookUpEditRecalculateMethod.Properties.DisplayMember = "Item2";
			lookUpEditRecalculateMethod.Properties.ValueMember = "Item1";

			lookUpEditRecalculateMethod.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			List<Tuple<enumPriceRounding, string>> priceRoundings = new List<Tuple<enumPriceRounding, string>>();
			priceRoundings.Add(new Tuple<enumPriceRounding, string>(enumPriceRounding._0_01, enumPriceRounding._0_01.GetAttributeDescription<EnumPolishName>()));
			priceRoundings.Add(new Tuple<enumPriceRounding, string>(enumPriceRounding._0_10, enumPriceRounding._0_10.GetAttributeDescription<EnumPolishName>()));
			priceRoundings.Add(new Tuple<enumPriceRounding, string>(enumPriceRounding._1_00, enumPriceRounding._1_00.GetAttributeDescription<EnumPolishName>()));
			priceRoundings.Add(new Tuple<enumPriceRounding, string>(enumPriceRounding._10_00, enumPriceRounding._10_00.GetAttributeDescription<EnumPolishName>()));
			priceRoundings.Add(new Tuple<enumPriceRounding, string>(enumPriceRounding._100_00, enumPriceRounding._100_00.GetAttributeDescription<EnumPolishName>()));
			priceRoundings.Add(new Tuple<enumPriceRounding, string>(enumPriceRounding._0_001, enumPriceRounding._0_001.GetAttributeDescription<EnumPolishName>()));
			priceRoundings.Add(new Tuple<enumPriceRounding, string>(enumPriceRounding._0_0001, enumPriceRounding._0_0001.GetAttributeDescription<EnumPolishName>()));

			lookUpEditRounding.Properties.DataSource = priceRoundings;
			lookUpEditRounding.Properties.DisplayMember = "Item2";
			lookUpEditRounding.Properties.ValueMember = "Item1";
			lookUpEditRounding.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			List<Tuple<enumProductType, string>> productTypes = new List<Tuple<enumProductType, string>>();
			productTypes.Add(new Tuple<enumProductType, string>(enumProductType.Article, enumProductType.Article.GetAttributeDescription<EnumPolishName>()));
			productTypes.Add(new Tuple<enumProductType, string>(enumProductType.Service, enumProductType.Service.GetAttributeDescription<EnumPolishName>()));
			productTypes.Add(new Tuple<enumProductType, string>(enumProductType.Kit, enumProductType.Kit.GetAttributeDescription<EnumPolishName>()));
			productTypes.Add(new Tuple<enumProductType, string>(enumProductType.Set, enumProductType.Set.GetAttributeDescription<EnumPolishName>()));
			productTypes.Add(new Tuple<enumProductType, string>(enumProductType.ServiceProfit, enumProductType.ServiceProfit.GetAttributeDescription<EnumPolishName>()));
			lookUpEditType.Properties.DataSource = productTypes;
			lookUpEditType.Properties.DisplayMember = "Item2";
			lookUpEditType.Properties.ValueMember = "Item1";
			lookUpEditType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			product.SalePrices.Remove(null);
			gridControlSalePrices.DataSource = product.SalePrices;
			gridControlSalePrices.RefreshDataSource();

			gridViewSalePrices.PopulateColumns();
			gridControlService.SetLocalization(ref gridViewSalePrices, localizations);

			GridColumn columnType = gridViewSalePrices.Columns["Type"];
			columnType.OptionsColumn.ReadOnly = true;
			columnType.OptionsColumn.AllowEdit = false;

			List<PriceTypeModel> ptModels = new List<PriceTypeModel>();
			foreach (enumSalePriceType pt in (enumSalePriceType[])Enum.GetValues(typeof(enumSalePriceType)))
				ptModels.Add(new PriceTypeModel() { Type = pt, Code = pt.GetAttributeDescription<EnumPolishName>() });

			gridControlService.AddLookupEdit(ref columnType, ptModels, nameof(PriceTypeModel.Type), nameof(PriceTypeModel.Code));

			var _currencies = currenciesRepository.GetStatic();
			if (_currencies.IsSucceed)
			{
				GridColumn columnCurrency = gridViewSalePrices.Columns["Currency"];
				gridControlService.AddLookupEdit(ref columnCurrency, _currencies.Object, nameof(Currency.Shortcut), nameof(Currency.Shortcut));
				RepositoryItemLookUpEdit ri = columnCurrency.ColumnEdit as RepositoryItemLookUpEdit;
				ri.PopulateColumns();
				ri.Columns["Id"].Visible = false;
				ri.Columns["Name"].Visible = false;
				ri.Columns["Active"].Visible = false;
				ri.Columns["Shortcut"].Caption = "Kod";
			}
			else
				messagesService.AddMessage(_currencies);

			List<Tuple<enumDefaultUnitOfMeasurementKind, string>> umKind = new List<Tuple<enumDefaultUnitOfMeasurementKind, string>>();
			umKind.Add(new Tuple<enumDefaultUnitOfMeasurementKind, string>(enumDefaultUnitOfMeasurementKind.None, enumDefaultUnitOfMeasurementKind.None.GetAttributeDescription<EnumPolishName>()));
			umKind.Add(new Tuple<enumDefaultUnitOfMeasurementKind, string>(enumDefaultUnitOfMeasurementKind.Record, enumDefaultUnitOfMeasurementKind.Record.GetAttributeDescription<EnumPolishName>()));
			umKind.Add(new Tuple<enumDefaultUnitOfMeasurementKind, string>(enumDefaultUnitOfMeasurementKind.Additional1, enumDefaultUnitOfMeasurementKind.Additional1.GetAttributeDescription<EnumPolishName>()));
			umKind.Add(new Tuple<enumDefaultUnitOfMeasurementKind, string>(enumDefaultUnitOfMeasurementKind.Additional2, enumDefaultUnitOfMeasurementKind.Additional2.GetAttributeDescription<EnumPolishName>()));
			lookUpEditDefault.Properties.DataSource = umKind;
			lookUpEditDefault.Properties.DisplayMember = "Item2";
			lookUpEditDefault.Properties.ValueMember = "Item1";
			lookUpEditDefault.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Rodzaj"));

			var _ums = unitsOfMeasurementRepository.GetStatic();
			if (_ums.IsSucceed)
			{
				lookUpEditAdditional1.Properties.DataSource = _ums.Object;
				lookUpEditAdditional1.Properties.DisplayMember = "Name";
				lookUpEditAdditional1.Properties.ValueMember = "Name";

				lookUpEditAdditional1.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Kod"));
				lookUpEditAdditional1.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Description", "Nazwa"));

				lookUpEditAdditional2.Properties.DataSource = _ums.Object;
				lookUpEditAdditional2.Properties.DisplayMember = "Name";
				lookUpEditAdditional2.Properties.ValueMember = "Name";

				lookUpEditAdditional2.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Kod"));
				lookUpEditAdditional2.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Description", "Nazwa"));

				lookUpEditRecord.Properties.DataSource = _ums.Object;
				lookUpEditRecord.Properties.DisplayMember = "Name";
				lookUpEditRecord.Properties.ValueMember = "Name";

				lookUpEditRecord.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Kod"));
				lookUpEditRecord.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Description", "Nazwa"));
			}
			else
				messagesService.AddMessage(_ums);

			if (product.Id > 0)
			{
				textEditCode.ReadOnly = true;
				lookUpEditType.ReadOnly = true;
			}
		}
		private void intitializeBinding()
		{
			textEditCode.DataBindings.Add("EditValue", product, nameof(product.Code), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditName.DataBindings.Add("EditValue", product, nameof(product.Name), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditActive.DataBindings.Add("EditValue", product, nameof(product.Active), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditType.DataBindings.Add("EditValue", product, nameof(product.Type), true, DataSourceUpdateMode.OnPropertyChanged);

			calcEditPurchasePrice.DataBindings.Add("EditValue", product, nameof(product.PurchasePrice) + "." + nameof(product.PurchasePrice.Value), true, DataSourceUpdateMode.OnPropertyChanged);
			calcEditBasePrice.DataBindings.Add("EditValue", product, nameof(product.BasePrice) + "." + nameof(product.BasePrice.Value), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditSettlementMethod.DataBindings.Add("EditValue", product, nameof(product.SettlementMethod), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditRecalculateMethod.DataBindings.Add("EditValue", product, nameof(product.PriceRecalculateType), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditRounding.DataBindings.Add("EditValue", product, nameof(product.PriceRounding), true, DataSourceUpdateMode.OnPropertyChanged);
			checkEditNegotiation.DataBindings.Add("EditValue", product, nameof(product.PriceNegotiation), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditDefault.DataBindings.Add("EditValue", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.DefaultUMKind), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditRecord.DataBindings.Add("EditValue", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.RecordUM), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditAdditional1.DataBindings.Add("EditValue", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.AdditionalUM1), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditAdditional2.DataBindings.Add("EditValue", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.AdditionalUM2), true, DataSourceUpdateMode.OnPropertyChanged);

			labelControlRecord1.DataBindings.Add("Text", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.RecordUM), true, DataSourceUpdateMode.OnPropertyChanged);
			labelControlRecord2.DataBindings.Add("Text", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.RecordUM), true, DataSourceUpdateMode.OnPropertyChanged);
			calcEditRatio1.DataBindings.Add("EditValue", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.RatioOfAdditionalUM1), true, DataSourceUpdateMode.OnPropertyChanged);
			calcEditRatio2.DataBindings.Add("EditValue", product, nameof(product.UnitsOfMeasurement) + "." + nameof(product.UnitsOfMeasurement.RatioOfAdditionalUM2), true, DataSourceUpdateMode.OnPropertyChanged);
		}
		private void refresh()
		{
			textEditCode.DataBindings.Clear();
			textEditName.DataBindings.Clear();
			checkEditActive.DataBindings.Clear();
			lookUpEditType.DataBindings.Clear();

			calcEditPurchasePrice.DataBindings.Clear();
			calcEditBasePrice.DataBindings.Clear();
			lookUpEditSettlementMethod.DataBindings.Clear();
			lookUpEditRecalculateMethod.DataBindings.Clear();
			lookUpEditRounding.DataBindings.Clear();
			checkEditNegotiation.DataBindings.Clear();

			lookUpEditDefault.DataBindings.Clear();
			lookUpEditRecord.DataBindings.Clear();
			lookUpEditAdditional1.DataBindings.Clear();
			lookUpEditAdditional2.DataBindings.Clear();

			labelControlRecord1.DataBindings.Clear();
			labelControlRecord2.DataBindings.Clear();
			calcEditRatio1.DataBindings.Clear();
			calcEditRatio2.DataBindings.Clear();

			intitializeBinding();

			gridControlSalePrices.DataSource = product.SalePrices;
			gridControlSalePrices.RefreshDataSource();
		}

		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "Type", "Typ" },
			{ "ProfitPercent", "Marża %" },
			{ "MarkupPercent", "Narzut %" },
			{ "LinkedWithBasePrice", "Powiązana z ceną bazową" },
			{ "AutomaticValue", "Wartość automatyczna" },
			{ "PriceParameter", "Parametr ceny" },
			{ "Value", "Cena" },
			{ "Currency", "Waluta" },
		};
	}
}
