﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Unity;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Demo.Model;
using DevExpress.XtraEditors;
using Unity.Resolution;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using Sage.PL.WebAPI.Interface.Products.ViewModels;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Products.ViewModels.Prices;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlProductsIndex : ManagedControl
	{
		public override Guid Identifier => new Guid("05FAE9F8-E878-4A4E-84FC-C077464A471C");

		private readonly IUnityContainer unityContainer;
		private readonly GridControlService gridControlService;
		private readonly DockManagerService dockManagerService;
		private readonly MessagesService messagesService;
		private readonly PricesRepository pricesRepository;
		private readonly ProductsRepository productsRepository;
		private readonly ProductMapper productMapper;

		private List<ProductModel> products;

		public ControlProductsIndex(IUnityContainer unityContainer, GridControlService gridControlService, DockManagerService dockManagerService, MessagesService messagesService,
			PricesRepository pricesRepository, ProductsRepository productsRepository, ProductMapper productMapper)
		{
			this.unityContainer = unityContainer;
			this.gridControlService = gridControlService;
			this.dockManagerService = dockManagerService;
			this.messagesService = messagesService;
			this.pricesRepository = pricesRepository;
			this.productsRepository = productsRepository;
			this.productMapper = productMapper;
			InitializeComponent();
			Text = "Kartoteka towarów";
		}

		private void ControlProductsIndex_Load(object sender, EventArgs e)
		{
			try
			{
				barStaticItemGuid.Caption = unityContainer.Resolve<string>("SessionGuid");
				loadData();
				gridViewProducts.PopulateColumns();

				gridControlService.SetColumnReadOnly(ref gridViewProducts, true);
				gridControlService.SetLocalization(ref gridViewProducts, localizations);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemEdit_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				int handle = gridViewProducts.FocusedRowHandle;
				if (handle < 0)
					return;

				var indexProduct = gridViewProducts.GetRow(handle) as ProductModel;
				var response = productsRepository.Get(indexProduct.Id);
				if (response.IsSucceed)
				{
					var control = unityContainer.Resolve<ControlProduct>(new ParameterOverride("product", response.Object));
					dockManagerService.OpenPanel(control);
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemNew_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var response = pricesRepository.GetSalePriceTypesStatic();
				if (response.IsSucceed)
				{
					Product product = new Product()
					{
						UnitsOfMeasurement = new ProductUnitsOfMeasurement(),
						PurchasePrice = new ProductPrice(),
						BasePrice = new ProductBasePrice(),
						SalePrices = new List<ProductSalePrice>()
					};

					foreach (var price in response.Object.Where(x => x.Active))
						product.SalePrices.Add(new ProductSalePrice() { Type = (enumSalePriceType)price.Type, Kind = price.Kind });

					var control = unityContainer.Resolve<ControlProduct>(new ParameterOverride("product", product));
					dockManagerService.OpenPanel(control);
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemRefresh_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));
				loadData();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void gridViewProducts_DoubleClick(object sender, EventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				GridView gridView = sender as GridView;
				if (gridView == null)
					return;

				int handle = gridView.FocusedRowHandle;
				if (handle < 0)
					return;

				var indexProduct = gridView.GetRow(handle) as ProductModel;
				var response = productsRepository.Get(indexProduct.Id);
				if (response.IsSucceed)
				{
					var control = unityContainer.Resolve<ControlProduct>(new ParameterOverride("product", response.Object));
					dockManagerService.OpenPanel(control);
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void loadData()
		{
			var response = productsRepository.Get();
			if (response.IsSucceed)
			{
				var _products = response.Object;
				products = productMapper.Map(_products);
				gridControlProducts.DataSource = products;
				gridControlProducts.RefreshDataSource();
			}
			else
				messagesService.AddMessage(response);
		}
		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "Code", "Kod" },
			{ "Name", "Nazwa" },
			{ "Type", "Typ" },
			{ "Active", "Aktywny" },
			{ "Unit", "Jednostka miary" },
			{ "VatRate", "Stawka VAT" },
		};
	}
}
