﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;
using DevExpress.XtraEditors;
using Sage.PL.WebAPI.Demo.ViewModels;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Interface.Common.ViewModels;
using DevExpress.XtraEditors.Repository;
using DevExpress.XtraGrid.Views.BandedGrid;
using DevExpress.XtraGrid.Columns;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Demo.Extensions;
using Sage.PL.WebAPI.Demo.Model;
using Sage.PL.WebAPI.Demo.Mappers;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlPurchaseDocument : ManagedControl
	{
		public override Guid Identifier => new Guid("28E0E9A4-EAC5-4329-83C8-4E9E9D879C3D");

		private readonly PurchaseDocumentRepository purchaseDocumentRepository;
		private readonly ContractorsRepository contractorsRepository;
		private readonly ProductsRepository productsRepository;
		private readonly PaymentRegistriesRepository paymentRegistriesRepository;
		private readonly PaymentFormsRepository paymentFormsRepository;
		private readonly DepartmentsRepository departmentsRepository;
		private readonly PurchaseDocumentMapper purchaseDocumentMapper;
		private readonly GridControlService gridControlService;
		private readonly MessagesService messagesService;
		private PurchaseDocumentViewModel purchaseDocument;

		public ControlPurchaseDocument(PurchaseDocumentRepository purchaseDocumentRepository, ContractorsRepository contractorsRepository,
			ProductsRepository productsRepository, PaymentRegistriesRepository paymentRegistriesRepository, PaymentFormsRepository paymentFormsRepository,
			DepartmentsRepository departmentsRepository, PurchaseDocumentMapper purchaseDocumentMapper,
			GridControlService gridControlService, MessagesService messagesService)
		{
			InitializeComponent();
			Text = "Dokument zakupu";

			purchaseDocument = new PurchaseDocumentViewModel()
			{
				Seller = new Interface.Contractors.ViewModels.ContractorListElement(),
				Deliverer = new Interface.Contractors.ViewModels.ContractorListElement(),
				DocumentType = new DocumentType(),
				DocumentSeries = new DocumentSeries(),
				PaymentForm = new PaymentForm(),
				PaymentRegistry = new PaymentRegistry(),
				Department = new Department(),
				IssueDate = DateTime.Now,
				MaturityDate = DateTime.Now,
				PurchaseDate = DateTime.Now,
				ReceiveDate = DateTime.Now,
				ForeignDocumentDate = DateTime.Now,
				Positions = new List<PurchaseDocumentPositionViewModel>()

			};
			this.purchaseDocumentRepository = purchaseDocumentRepository;
			this.contractorsRepository = contractorsRepository;
			this.productsRepository = productsRepository;
			this.paymentRegistriesRepository = paymentRegistriesRepository;
			this.paymentFormsRepository = paymentFormsRepository;
			this.departmentsRepository = departmentsRepository;
			this.purchaseDocumentMapper = purchaseDocumentMapper;
			this.gridControlService = gridControlService;
			this.messagesService = messagesService;
			intitializeBinding();
		}

		private void ControlForeignOrder_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemSave_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var purchaseDocumentIssue = purchaseDocumentMapper.Map(purchaseDocument);
				var _purchaseDocument = purchaseDocumentRepository.AddNew(purchaseDocumentIssue);
				if (_purchaseDocument.IsSucceed)
					XtraMessageBox.Show("Zapisano nowy dokument zakupu");
				else
					messagesService.AddMessage(_purchaseDocument);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonAddPosition_Click(object sender, EventArgs e)
		{
			try
			{
				purchaseDocument.Positions.Add(new PurchaseDocumentPositionViewModel() { No = purchaseDocument.Positions.Count + 1 });
				gridControlPositions.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonDelete_Click(object sender, EventArgs e)
		{
			try
			{
				var handle = advBandedGridViewPositions.FocusedRowHandle;
				if (handle < 0)
					return;

				PurchaseDocumentPositionViewModel position = advBandedGridViewPositions.GetRow(handle) as PurchaseDocumentPositionViewModel;
				if (position == null)
					return;

				purchaseDocument.Positions.Remove(position);
				int no = 1;
				purchaseDocument.Positions.ForEach(x => x.No = no++);
				gridControlPositions.RefreshDataSource();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}

		private void advBandedGridViewPositions_CellValueChanged(object sender, DevExpress.XtraGrid.Views.Base.CellValueChangedEventArgs e)
		{
			try
			{
				AdvBandedGridView gridView = sender as AdvBandedGridView;
				if (gridView == null)
					return;

				int handle = gridView.FocusedRowHandle;
				if (handle < 0)
					return;

				var position = gridView.GetRow(handle) as PurchaseDocumentPositionViewModel;
				if (position == null)
					return;

				GridColumn column = gridView.FocusedColumn;
				if (column.FieldName.Equals(nameof(PurchaseDocumentPositionViewModel.Qunatity)) || column.FieldName.Equals(nameof(PurchaseDocumentPositionViewModel.Price)))
					gridControlPositions.RefreshDataSource();
			}
			catch (Exception exc)
			{
			}
		}
		private void lookUpEditPaymentRegistry_EditValueChanged(object sender, EventArgs e)
		{
			try
			{
				var registry = lookUpEditPaymentRegistry.EditValue as PaymentRegistry;
				if (registry?.Id != null)
				{
					var paymentForms = paymentFormsRepository.Get(registry.Id.Value);
					if (paymentForms.IsSucceed)
					{
						var payment = lookUpEditPaymentForm.EditValue as PaymentForm;
						if (!paymentForms.Object.Exists(x => x.Id == payment?.Id))
							purchaseDocument.PaymentForm = null;

						lookUpEditPaymentForm.Properties.DataSource = paymentForms.Object;
					}
				}
			}
			catch (Exception exc)
			{
			}
		}
		private void lookUpEditDocumentType_EditValueChanged(object sender, EventArgs e)
		{
			try
			{
				var type = lookUpEditDocumentType.EditValue as DocumentType;
				if (type?.Id != null)
				{
					var _serie = purchaseDocumentRepository.GetDocumentSeries(type.Id);
					if (_serie.IsSucceed)
					{
						var serie = lookUpEditDocumentSerie.EditValue as DocumentSeries;
						if (!_serie.Object.Exists(x => x.Id == serie?.Id))
							purchaseDocument.DocumentSeries = null;

						lookUpEditDocumentSerie.Properties.DataSource = _serie.Object;
					}
					else
						messagesService.AddMessage(_serie);
				}
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}

		private void initialize()
		{
			var paymentRegistries = paymentRegistriesRepository.GetStatic();
			if (paymentRegistries.IsSucceed)
			{
				lookUpEditPaymentRegistry.Properties.DataSource = paymentRegistries.Object;
				lookUpEditPaymentRegistry.Properties.DisplayMember = "Name";
				lookUpEditPaymentRegistry.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(paymentRegistries);

			lookUpEditPaymentForm.Properties.DisplayMember = "Name";
			lookUpEditPaymentForm.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			lookUpEditPaymentForm.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Days", "Dni"));

			var contractors = contractorsRepository.Get();
			if (contractors.IsSucceed)
			{
				lookUpEditSeller.Properties.DataSource = contractors.Object;
				lookUpEditSeller.Properties.DisplayMember = "Code";
				lookUpEditSeller.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditSeller.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));

				lookUpEditDeliverer.Properties.DataSource = contractors.Object;
				lookUpEditDeliverer.Properties.DisplayMember = "Code";
				lookUpEditDeliverer.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditDeliverer.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(contractors);

			var types = purchaseDocumentRepository.GetFVZDocumentTypes();
			if (types.IsSucceed)
			{
				lookUpEditDocumentType.Properties.DataSource = types.Object;
				lookUpEditDocumentType.Properties.DisplayMember = "Code";
				lookUpEditDocumentType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditDocumentType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(types);

			lookUpEditDocumentSerie.Properties.DisplayMember = "Code";
			lookUpEditDocumentSerie.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));

			var depart = departmentsRepository.GetStatic();
			if (depart.IsSucceed)
			{
				lookUpEditDepartment.Properties.DataSource = depart.Object;
				lookUpEditDepartment.Properties.DisplayMember = "Code";
				lookUpEditDepartment.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
				lookUpEditDepartment.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Name", "Nazwa"));
			}
			else
				messagesService.AddMessage(depart);

			var _products = productsRepository.Get();
			if (_products.IsSucceed)
			{
				var columnProduct = advBandedGridViewPositions.Columns["Product"];
				gridControlService.AddLookupEdit(ref columnProduct, _products.Object, "", "Code");

				RepositoryItemLookUpEdit ri = columnProduct.ColumnEdit as RepositoryItemLookUpEdit;
				if (ri != null)
				{
					ri.PopulateColumns();
					ri.Columns["Id"].Visible = false;
					ri.Columns["Active"].Visible = false;
					ri.Columns["Unit"].Visible = false;
					ri.Columns["DefaultUnit"].Visible = false;
					ri.Columns["VatRate"].Visible = false;
					ri.Columns["Barcode"].Visible = false;
					ri.Columns["CN"].Visible = false;
					ri.Columns["PKWiU"].Visible = false;
					ri.Columns["FiscalName"].Visible = false;

					ri.Columns["Code"].Caption = "Kod";
					ri.Columns["Name"].Caption = "Nazwa";
					ri.Columns["Type"].Caption = "Typ";

					ri.NullText = string.Empty;
				}
			}
			else
				messagesService.AddMessage(_products);

			List<Tuple<enumPriceKind, string>> priceKinds = new List<Tuple<enumPriceKind, string>>();
			priceKinds.Add(new Tuple<enumPriceKind, string>(enumPriceKind.Net, enumPriceKind.Net.GetAttributeDescription<EnumPolishName>()));
			priceKinds.Add(new Tuple<enumPriceKind, string>(enumPriceKind.Gross, enumPriceKind.Gross.GetAttributeDescription<EnumPolishName>()));

			lookUpEditPriceKind.Properties.DataSource = priceKinds;
			lookUpEditPriceKind.Properties.DisplayMember = "Item2";
			lookUpEditPriceKind.Properties.ValueMember = "Item1";
			lookUpEditPriceKind.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));

			gridControlPositions.DataSource = purchaseDocument.Positions;
			gridControlPositions.RefreshDataSource();
		}
		private void intitializeBinding()
		{
			lookUpEditDeliverer.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.Deliverer), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditSeller.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.Seller), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPaymentRegistry.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.PaymentRegistry), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditPaymentForm.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.PaymentForm), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditSeller.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.SellerName), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditDeliverer.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.DelivererName), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditSellerCity.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.Seller) + "." + nameof(purchaseDocument.Seller.Place), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditDelivererCity.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.Deliverer) + "." + nameof(purchaseDocument.Deliverer.Place), true, DataSourceUpdateMode.OnPropertyChanged);

			textEditDescription.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.Description), true, DataSourceUpdateMode.OnPropertyChanged);
			textEditForeignDocumentNumber.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.ForeignDocumentNumber), true, DataSourceUpdateMode.OnPropertyChanged);
			richTextBoxNote.DataBindings.Add("Text", purchaseDocument, nameof(purchaseDocument.Note), true, DataSourceUpdateMode.OnPropertyChanged);

			dateEditIssueDate.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.IssueDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditPurchaseDate.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.PurchaseDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditMaturityDate.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.MaturityDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditForeignDocumentDate.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.ForeignDocumentDate), true, DataSourceUpdateMode.OnPropertyChanged);
			dateEditReceiveDate.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.ReceiveDate), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditDocumentType.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.DocumentType), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditDocumentSerie.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.DocumentSeries), true, DataSourceUpdateMode.OnPropertyChanged);
			lookUpEditDepartment.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.Department), true, DataSourceUpdateMode.OnPropertyChanged);

			lookUpEditPriceKind.DataBindings.Add("EditValue", purchaseDocument, nameof(purchaseDocument.PriceKind), true, DataSourceUpdateMode.OnPropertyChanged);
		}
	}
}
