﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Sage.PL.WebAPI.Demo.Services;
using Unity;
using Sage.PL.WebAPI.Demo.Mappers;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Demo.Model;
using DevExpress.XtraGrid.Columns;
using System.IO;
using DevExpress.XtraEditors;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.Forms;

namespace Sage.PL.WebAPI.Demo.Controls
{
	public partial class ControlPurchaseDocumentsIndex : ManagedControl
	{
		public override Guid Identifier => new Guid("0CF8D609-A370-496F-8530-AA6F7E4438A5");

		private readonly IUnityContainer unityContainer;
		private readonly GridControlService gridControlService;
		private readonly DockManagerService dockManagerService;
		private readonly MessagesService messagesService;
		private readonly PurchaseDocumentRepository purchaseDocumentRepository;
		private readonly PurchaseDocumentMapper purchaseDocumentMapper;
		private readonly ContractorsRepository contractorsRepository;
		private readonly ContractorMapper contractorMapper;

		private List<PurchaseDocumentModel> purchaseDocuments;
		private List<ContractorModel> contractors;

		public ControlPurchaseDocumentsIndex(IUnityContainer unityContainer, GridControlService gridControlService, DockManagerService dockManagerService, MessagesService messagesService,
			PurchaseDocumentRepository purchaseDocumentRepository, PurchaseDocumentMapper purchaseDocumentMapper,
			ContractorsRepository contractorsRepository, ContractorMapper contractorMapper)
		{
			this.unityContainer = unityContainer;
			this.gridControlService = gridControlService;
			this.dockManagerService = dockManagerService;
			this.messagesService = messagesService;
			this.purchaseDocumentRepository = purchaseDocumentRepository;
			this.purchaseDocumentMapper = purchaseDocumentMapper;
			this.contractorsRepository = contractorsRepository;
			this.contractorMapper = contractorMapper;
			InitializeComponent();
			Text = "Kartoteka dokumentów zakupu";
		}

		private void ControlPurchaseDocumentsIndex_Load(object sender, EventArgs e)
		{
			try
			{
				barStaticItemGuid.Caption = unityContainer.Resolve<string>("SessionGuid");
				loadStaticData();
				loadData();
				gridViewPurchaseDocuments.PopulateColumns();

				gridControlService.SetColumnReadOnly(ref gridViewPurchaseDocuments, true);
				gridControlService.SetLocalization(ref gridViewPurchaseDocuments, localizations);

				GridColumn column;
				column = gridViewPurchaseDocuments.Columns[nameof(PurchaseDocumentModel.SellerId)];
				if (column != null)
					gridControlService.AddLookupEdit(ref column, contractors);
				column = gridViewPurchaseDocuments.Columns[nameof(PurchaseDocumentModel.DelivererId)];
				if (column != null)
					gridControlService.AddLookupEdit(ref column, contractors);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemNew_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlPurchaseDocument>();
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemPDF_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var handle = gridViewPurchaseDocuments.FocusedRowHandle;
				if (handle < 0)
					return;

				PurchaseDocumentModel purchaseDocument = gridViewPurchaseDocuments.GetRow(handle) as PurchaseDocumentModel;
				if (purchaseDocument == null)
					return;

				var message = purchaseDocumentRepository.GetPDF(purchaseDocument.Id);
				if (message.IsSucceed)
				{
					var pdf = message.Object;
					using (SaveFileDialog sfd = new SaveFileDialog())
					{
						sfd.Filter = "pliki PDF (*.pdf)|*.pdf";
						sfd.RestoreDirectory = true;
						sfd.FileName = purchaseDocument.DocumentNumber.Replace(@"/", "_");

						if (sfd.ShowDialog() == DialogResult.OK)
						{
							var pdfBytes = Convert.FromBase64String(pdf.ContentFile_Base64);
							File.WriteAllBytes(sfd.FileName, pdfBytes);
						}
					}
				}
				else
					messagesService.AddMessage(message);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemRefresh_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				loadData();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void loadData()
		{
			DateTime? dateFrom = (DateTime?)barEditItemDateFrom.EditValue;
			DateTime? dateTo = (DateTime?)barEditItemDateTo.EditValue;

			var message = purchaseDocumentRepository.GetList(dateFrom, dateTo);
			if (message.IsSucceed)
			{
				var _purchaseDocuments = message.Object;
				purchaseDocuments = purchaseDocumentMapper.Map(_purchaseDocuments);
				gridControlPurchaseDocuments.DataSource = purchaseDocuments;
				gridControlPurchaseDocuments.RefreshDataSource();
			}
			else
				messagesService.AddMessage(message);
		}
		private void loadStaticData()
		{
			var message = contractorsRepository.Get();
			if (message.IsSucceed)
			{
				var _contractors = message.Object;
				contractors = contractorMapper.Map(_contractors);
			}
			else
				messagesService.AddMessage(message);
		}
		private Dictionary<string, string> localizations = new Dictionary<string, string>()
		{
			{ "DocumentNumber", "Numer dokumentu" },
			{ "Active", "Aktywny" },
			{ "Buffer", "Bufor" },
			{ "Settled", "Rozliczony" },
			{ "IssueDate", "Data wystawienia" },
			{ "PurchaseDate", "Data zakupu" },
			{ "ReceiveDate", "Data wpływu" },
			{ "ForeignDocumentDate", "Data dokumentu obcego" },
			{ "MaturityDate", "Termin płatności" },
			{ "SellerId", "Sprzedawca" },
			{ "DelivererId", "Dostawca" },
			{ "NetValuePLN", "Wartość netto PLN" },
			{ "VatValuePLN", "Wartość VAT PLN" },
			{ "NetValue", "Wartość netto" },
			{ "GrossValue", "Wartość brutto" },
			{ "Currency", "Waluta" },
		};
	}
}
