﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DevExpress.Utils.Menu;
using DevExpress.XtraGrid.Menu;
using Sage.PL.WebAPI.Demo.Controls;
using Sage.PL.WebAPI.Demo.Model;

namespace Sage.PL.WebAPI.Demo.Extensions
{
	public class GridMenu : GridViewMenu
	{
		public GridMenu(DevExpress.XtraGrid.Views.Grid.GridView view) : base(view) { }

		// This method is automatically called by the menu's public Init method. 
		protected override void CreateItems()
		{
			Items.Clear();
			Items.Add(CreateMenuItem("Zaznacz wszystko", null, "SelectAll", true));
			Items.Add(CreateMenuItem("Odznacz wszystko", null, "SelectNothing", true));
			Items.Add(CreateMenuItem("Odwróć zaznaczenie", null, "SelectRevert", true));
		}

		protected override void OnMenuItemClick(object sender, EventArgs e)
		{
			DXMenuItem item = sender as DXMenuItem;
			if (item?.Tag == null)
				return;

			var list = View.DataSource as List<ListElementModel>;

			switch (item.Tag)
			{
				case "SelectAll":
					list.ForEach(x => x.Check = true);
					View.RefreshData();
					break;
				case "SelectNothing":
					list.ForEach(x => x.Check = false);
					View.RefreshData();
					break;
				case "SelectRevert":
					list.ForEach(x => x.Check = !x.Check);
					View.RefreshData();
					break;
				default:
					break;
			}
		}
	}

	public class GridMenuRandom : GridMenu
	{
		public GridMenuRandom(DevExpress.XtraGrid.Views.Grid.GridView view) : base(view) { }

		protected override void CreateItems()
		{
			base.CreateItems();
			Items.Add(CreateMenuItem("Losuj ilość", null, "Random", true));
		}

		protected override void OnMenuItemClick(object sender, EventArgs e)
		{
			DXMenuItem item = sender as DXMenuItem;
			if (item?.Tag == null)
				return;

			var list = View.DataSource as List<ListElementExtModel>;
			switch (item.Tag)
			{
				case "SelectAll":
					list.ForEach(x => x.Check = true);
					View.RefreshData();
					break;
				case "SelectNothing":
					list.ForEach(x => x.Check = false);
					View.RefreshData();
					break;
				case "SelectRevert":
					list.ForEach(x => x.Check = !x.Check);
					View.RefreshData();
					break;
				case "Random":
					ControlRandomQuantity control = new ControlRandomQuantity();
					if (DevExpress.XtraEditors.XtraDialog.Show(control, "Losuj ilość", MessageBoxButtons.OK) == DialogResult.OK)
					{
						Random r = new Random();

						list.ForEach(x =>
						{
							if (x.Check)
								x.Quantity = r.Next(control.QuantityFrom, control.QuantityTo);
							else
								x.Quantity = 1m;
						});
						View.RefreshData();
					}
					break;
				default:
					break;
			}
		}
	}
}