﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DevExpress.XtraEditors;
using DevExpress.XtraSplashScreen;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using Sage.PL.WebAPI.Demo.Services;
using Unity;

namespace Sage.PL.WebAPI.Demo.Forms
{
	public partial class FormLogin : XtraForm
	{
		private readonly IUnityContainer unityContainer;
		private readonly Bootstrap.HttpClientFactory httpClientFactory;
		private readonly MessagesService messagesService;

		public bool FK { get; private set; }

		public FormLogin(IUnityContainer unityContainer, Bootstrap.HttpClientFactory httpClientFactory, MessagesService messagesService)
		{
			InitializeComponent();
			initialize();

			this.unityContainer = unityContainer;
			this.httpClientFactory = httpClientFactory;
			this.messagesService = messagesService;
		}
		private void initialize()
		{
			List<Tuple<int, string>> types = new List<Tuple<int, string>>()
			{
				new Tuple<int, string>(1, "Handel"),
				new Tuple<int, string>(2, "Finanse i Księgowość")
			};

			lookUpEditType.Properties.DataSource = types;
			lookUpEditType.Properties.DisplayMember = "Item2";
			lookUpEditType.Properties.ValueMember = "Item1";

			lookUpEditType.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Item2", "Typ"));
			lookUpEditType.EditValue = 1;
		}

		private void simpleButtonPing_Click(object sender, EventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				Uri uri = new Uri(textEditAddress.Text);
				httpClientFactory.Temporary(uri);

				PingRepository pingRepository = unityContainer.Resolve<PingRepository>();
				var response = pingRepository.Get();
				if (response.IsSucceed)
				{
					SplashScreenManager.CloseForm(false);

					using (FormPing formPing = unityContainer.Resolve<FormPing>())
					{
						formPing.Initialize(response.Object);
						formPing.ShowDialog();
					}
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonAlive_Click(object sender, EventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				Uri uri = new Uri(textEditAddress.Text);
				httpClientFactory.Temporary(uri);

				AliveRepository aliveRepository = unityContainer.Resolve<AliveRepository>();
				var response = aliveRepository.Get();
				if (response.IsSucceed)
				{
					SplashScreenManager.CloseForm(false);
					XtraMessageBox.Show(response.Object.ToString(), "Alive");
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonLogin_Click(object sender, EventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));
				Uri uri = new Uri(textEditAddress.Text);
				httpClientFactory.Temporary(uri);
				httpClientFactory.InitializeAppGuid(textEditGuid.Text);
				httpClientFactory.InitializeSessionGuid(string.Empty);

				SessionRepository sessionRepository = unityContainer.Resolve<SessionRepository>();
				var response = sessionRepository.OpenNewSession("WebAPI Demo");
				if (response.IsSucceed)
				{
					httpClientFactory.Initialize(uri);
					httpClientFactory.InitializeSessionGuid(response.Object);
					FK = (int)lookUpEditType.EditValue == 2;
					DialogResult = DialogResult.OK;
				}
				else
					messagesService.AddMessage(response);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonExit_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
		}
	}
}
