﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using DevExpress.XtraEditors;
using System.ComponentModel.DataAnnotations;
using System.IO;
using DevExpress.XtraLayout.Helpers;
using DevExpress.XtraLayout;
using Sage.PL.WebAPI.Demo.Services;
using Unity;
using Sage.PL.WebAPI.Demo.Controls;
using DevExpress.XtraBars.Ribbon;
using Sage.PL.WebAPI.Demo.DAL.Repositories;
using DevExpress.XtraSplashScreen;

namespace Sage.PL.WebAPI.Demo.Forms
{
	public partial class FormMain : DevExpress.XtraBars.Ribbon.RibbonForm
	{
		private readonly IUnityContainer unityContainer;
		private readonly DockManagerService dockManagerService;
		private readonly SessionRepository sessionRepository;
		private readonly MessagesService messagesService;

		public FormMain(IUnityContainer unityContainer, DockManagerService dockManagerService,
			SessionRepository sessionRepository, MessagesService messagesService,
			bool fk)
		{
			this.unityContainer = unityContainer;
			this.dockManagerService = dockManagerService;
			this.sessionRepository = sessionRepository;
			this.messagesService = messagesService;
			InitializeComponent();
			dockManagerService.Initialize(dockManager);

			foreach (RibbonPage page in mainRibbonControl.Pages)
			{
				if (page.PageIndex % 2 == 0 == fk)
					page.Visible = false;
			}
		}

		private void barButtonItemHandelContractors_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var index = unityContainer.Resolve<ControlContractorsIndex>();
				dockManagerService.OpenPanel(index);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemHandelProducts_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var index = unityContainer.Resolve<ControlProductsIndex>();
				dockManagerService.OpenPanel(index);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemForeignOrders_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var index = unityContainer.Resolve<ControlForeignOrdersIndex>();
				dockManagerService.OpenPanel(index);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemPurchaseDocuments_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var index = unityContainer.Resolve<ControlPurchaseDocumentsIndex>();
				dockManagerService.OpenPanel(index);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void barButtonItemFKContractors_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var index = unityContainer.Resolve<ControlFKContractorsIndex>();
				dockManagerService.OpenPanel(index);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemFKEmployees_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var index = unityContainer.Resolve<ControlFKEmployeesIndex>();
				dockManagerService.OpenPanel(index);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemFKDocuments_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var index = unityContainer.Resolve<ControlFKDocumentsIndex>();
				dockManagerService.OpenPanel(index);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemHandelContractorTest_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlContractorTest>();
				control.FormMain = this;
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemHandelProductTest_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlProductTest>();
				control.FormMain = this;
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemFKContractorTest_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlFKContractorTest>();
				control.FormMain = this;
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemEmployeeTest_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlFKEmployeeTest>();
				control.FormMain = this;
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemHandelForeignOrderTest_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlForeignOrderTest>();
				control.FormMain = this;
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemDocumentTest_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlFKDocumentTest>();
				control.FormMain = this;
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}
		private void barButtonItemPrice_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
		{
			try
			{
				SplashScreenManager.ShowForm(typeof(FormWaiting));

				var control = unityContainer.Resolve<ControlPriceTest>();
				control.FormMain = this;
				dockManagerService.OpenPanel(control);
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				SplashScreenManager.CloseForm(false);
				messagesService.ShowMessages();
			}
		}

		private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
		{
			try
			{
				var message = sessionRepository.CloseSession();
			}
			catch
			{ }
		}

		public void SetEdit(bool mode)
		{
			mainRibbonControl.Enabled = mode;
		}

	}
}
