﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DevExpress.XtraEditors;
using Sage.PL.WebAPI.Demo.DAL.StaticRepositories;
using Sage.PL.WebAPI.Demo.Services;
using Sage.PL.WebAPI.Interface.FKF.Common.ViewModels;

namespace Sage.PL.WebAPI.Demo.Forms
{
	public partial class FormYear : Form
	{
		private readonly FKYearsRepository yearsRepository;
		private readonly MessagesService messagesService;

		public int YearId { get; private set; }

		public FormYear(FKYearsRepository yearsRepository, MessagesService messagesService)
		{
			this.yearsRepository = yearsRepository;
			this.messagesService = messagesService;
			InitializeComponent();
			initialize();
		}

		private void FormYear_Load(object sender, EventArgs e)
		{
			try
			{
				initialize();
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonOk_Click(object sender, EventArgs e)
		{
			try
			{
				YearId = Convert.ToInt32(lookUpEditYear.EditValue);
				DialogResult = DialogResult.OK;
			}
			catch (Exception exc)
			{
				messagesService.AddMessage(exc);
			}
			finally
			{
				messagesService.ShowMessages();
			}
		}
		private void simpleButtonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
		}

		private void initialize()
		{
			var _years = yearsRepository.GetStatic();
			if (_years.IsSucceed)
			{
				lookUpEditYear.Properties.ValueMember = nameof(Year.Id);
				lookUpEditYear.Properties.DisplayMember = nameof(Year.Code);
				lookUpEditYear.Properties.DataSource = _years.Object;
				lookUpEditYear.EditValue = _years.Object.Min(x => x.Id);

				lookUpEditYear.Properties.Columns.Add(new DevExpress.XtraEditors.Controls.LookUpColumnInfo("Code", "Kod"));
			}
			else
				messagesService.AddMessage(_years);
		}
	}
}
