﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Sage.PL.WebAPI.Demo.Extensions;
using Sage.PL.WebAPI.Demo.Model;
using Sage.PL.WebAPI.Interface.Attributes;
using Sage.PL.WebAPI.Interface.Contractors.ViewModels;
using Sage.PL.WebAPI.Interface.FKF.Contractors.ViewModels;
using Sage.PL.WebAPI.Interface.Enums;
using ContractorListElement = Sage.PL.WebAPI.Interface.Contractors.ViewModels.ContractorListElement;
using FKFContractorListElement = Sage.PL.WebAPI.Interface.FKF.Contractors.ViewModels.ContractorListElement;

namespace Sage.PL.WebAPI.Demo.Mappers
{
	public class ContractorMapper
	{
		public ContractorModel Map(ContractorListElement contractor)
		{
			if (contractor == null)
				return null;

			ContractorModel model = new ContractorModel();
			model.Id = contractor.Id;
			model.Code = contractor.Code;
			model.Name = contractor.Name;
			model.Type = contractor.Type?.GetAttributeDescription<EnumPolishName>();
			model.NIP = contractor.NIP;
			model.Active = contractor.Active;
			model.PriceType = contractor.PriceType?.GetAttributeDescription<EnumPolishName>();
			model.PriceKind = contractor.PriceKind?.GetAttributeDescription<EnumPolishName>();
			model.Place = contractor.Place;

			return model;
		}
		public List<ContractorModel> Map(IEnumerable<ContractorListElement> contractors)
		{
			if (contractors == null)
				return null;

			List<ContractorModel> list = new List<ContractorModel>();
			foreach (var contractor in contractors)
				list.Add(Map(contractor));

			return list;
		}

		public FKContractorModel Map(FKFContractorListElement contractor)
		{
			if (contractor == null)
				return null;

			FKContractorModel model = new FKContractorModel();
			model.Id = contractor.Id;
			model.Position = contractor.Position;
			model.Code = contractor.Code;
			model.Name = contractor.Name;
			model.Type = contractor.Type?.GetAttributeDescription<EnumPolishName>();
			model.NIP = contractor.NIP;
			model.Active = contractor.Active;
			model.Place = contractor.Place;

			return model;
		}
		public List<FKContractorModel> Map(IEnumerable<FKFContractorListElement> contractors)
		{
			if (contractors == null)
				return null;

			List<FKContractorModel> list = new List<FKContractorModel>();
			foreach (var contractor in contractors)
				list.Add(Map(contractor));

			return list;
		}
	}
}
