﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Sage.PL.WebAPI.Demo.Model;
using Sage.PL.WebAPI.Demo.ViewModels;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Orders.ViewModels;

namespace Sage.PL.WebAPI.Demo.Mappers
{
	public class ForeignOrderMapper
	{
		public ForeignOrderModel Map(OrderListElement order)
		{
			if (order == null)
				return null;

			ForeignOrderModel model = new ForeignOrderModel();
			model.Id = order.Id;
			model.DocumentNumber = order.DocumentNumber;
			model.GrossValue = order.GrossValue;
			model.NetValue = order.NetValue;
			model.VatValuePLN = order.VatValuePLN;
			model.NetValuePLN = order.NetValuePLN;
			model.RecipientId = order.RecipientId;
			model.BuyerId = order.BuyerId;
			model.Currency = order.Currency;
			model.MaturityDate = order.MaturityDate;
			model.SaleDate = order.SaleDate;
			model.IssueDate = order.IssueDate;
			model.Settled = order.Settled;
			model.Buffer = order.Buffer;
			model.Active = order.Active;

			return model;
		}
		public List<ForeignOrderModel> Map(IEnumerable<OrderListElement> orders)
		{
			if (orders == null)
				return null;

			List<ForeignOrderModel> list = new List<ForeignOrderModel>();
			foreach (var order in orders)
				list.Add(Map(order));

			return list;
		}

		public OrderIssue Map(OrderViewModel viewModel)
		{
			if (viewModel == null)
				return null;

			OrderIssue issue = new OrderIssue();
			issue.Buyer = new OrderIssueContractor() { Id = viewModel.Buyer?.Id };
			issue.Recipient = new OrderIssueContractor() { Id = viewModel.Recipient?.Id };

			issue.MaturityDate = viewModel.MaturityDate;
			issue.SaleDate = viewModel.SaleDate;
			issue.IssueDate = viewModel.IssueDate;

			issue.ReservationType = enumDocumentReservationType.None;
			issue.ReceivedBy = viewModel.RecivedBy;
			issue.Note = viewModel.Note;
			issue.Description = viewModel.Description;
			issue.SalePriceType = viewModel.SalePriceType;
			issue.PriceKind = viewModel.PriceKind;

			issue.Department = viewModel.Department?.Code;
			issue.PaymentRegistry = viewModel.PaymentRegistry;
			issue.PaymentFormId = viewModel.PaymentForm?.Id ?? 0;
			issue.TypeCode = viewModel.DocumentType?.Code;
			issue.Series = viewModel.DocumentType?.Code;

			issue.Positions = new List<OrderIssuePosition>();
			foreach (var positions in viewModel.Positions)
			{
				OrderIssuePosition issuePosition = new OrderIssuePosition();
				issuePosition.Code = positions.Product?.Code;
				issuePosition.Name = positions.Product?.Name;
				issuePosition.UnitOfMeasurement = positions.Product?.Unit;
				issuePosition.VatRate = new Interface.Common.ViewModels.VatRateBase() { Code = positions.Product?.VatRate };
				issuePosition.Quantity = positions.Qunatity;
				issuePosition.Value = positions.Value;

				issue.Positions.Add(issuePosition);
			}

			return issue;
		}
	}
}
