﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Sage.PL.WebAPI.Demo.Model;
using Sage.PL.WebAPI.Demo.ViewModels;
using Sage.PL.WebAPI.Interface.Enums;
using Sage.PL.WebAPI.Interface.Purchases.ViewModels;

namespace Sage.PL.WebAPI.Demo.Mappers
{
	public class PurchaseDocumentMapper
	{
		public PurchaseDocumentModel Map(PurchaseDocumentListElement order)
		{
			if (order == null)
				return null;

			PurchaseDocumentModel model = new PurchaseDocumentModel();
			model.Id = order.Id;
			model.DocumentNumber = order.DocumentNumber;
			model.GrossValue = order.GrossValue;
			model.NetValue = order.NetValue;
			model.VatValuePLN = order.VatValuePLN;
			model.NetValuePLN = order.NetValuePLN;
			model.SellerId = order.SellerId;
			model.DelivererId = order.DelivererId;
			model.Currency = order.Currency;
			model.MaturityDate = order.MaturityDate;
			model.PurchaseDate = order.PurchaseDate;
			model.IssueDate = order.IssueDate;
			model.ReceiveDate = order.ReceiveDate;
			model.ForeignDocumentDate = order.ForeignDocumentDate;
			model.Settled = order.Settled;
			model.Buffer = order.Buffer;
			model.Active = order.Active;

			return model;
		}
		public List<PurchaseDocumentModel> Map(IEnumerable<PurchaseDocumentListElement> orders)
		{
			if (orders == null)
				return null;

			List<PurchaseDocumentModel> list = new List<PurchaseDocumentModel>();
			foreach (var order in orders)
				list.Add(Map(order));

			return list;
		}

		public PurchaseDocumentIssue Map(PurchaseDocumentViewModel viewModel)
		{
			if (viewModel == null)
				return null;

			PurchaseDocumentIssue issue = new PurchaseDocumentIssue();
			issue.Seller = new PurchaseDocumentIssueContractor() { Id = viewModel.Seller?.Id };
			issue.Deliverer = new PurchaseDocumentIssueContractor() { Id = viewModel.Deliverer?.Id };

			issue.MaturityDate = viewModel.MaturityDate;
			issue.ReceiveDate = viewModel.ReceiveDate;
			issue.PurchaseDate = viewModel.PurchaseDate;
			issue.ForeignDocumentDate = viewModel.ForeignDocumentDate;
			issue.ForeignDocumentNumber = viewModel.ForeignDocumentNumber;
			issue.IssueDate = viewModel.IssueDate;

			issue.Note = viewModel.Note;
			issue.Description = viewModel.Description;
			issue.PriceKind = viewModel.PriceKind;

			issue.Department = viewModel.Department?.Code;
			issue.PaymentRegistry = viewModel.PaymentRegistry;
			issue.PaymentFormId = viewModel.PaymentForm?.Id ?? 0;
			issue.TypeCode = viewModel.DocumentType?.Code;
			issue.Series = viewModel.DocumentType?.Code;

			issue.Positions = new List<PurchaseDocumentIssuePosition>();
			foreach (var positions in viewModel.Positions)
			{
				PurchaseDocumentIssuePosition issuePosition = new PurchaseDocumentIssuePosition();
				issuePosition.Code = positions.Product?.Code;
				issuePosition.Name = positions.Product?.Name;
				issuePosition.UnitOfMeasurement = positions.Product?.Unit;
				issuePosition.VatRate = new Interface.Common.ViewModels.VatRateBase() { Code = positions.Product?.VatRate };
				issuePosition.Quantity = positions.Qunatity;
				issuePosition.Value = positions.Value;

				issue.Positions.Add(issuePosition);
			}

			return issue;
		}
	}
}
