﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DevExpress.XtraBars.Docking;
using DevExpress.XtraBars.Docking2010;
using DevExpress.XtraEditors;

namespace Sage.PL.WebAPI.Demo.Services
{
	public class DockManagerService
	{
		private DockManager dockManager;

		public void Initialize(DockManager dockManager)
		{
			this.dockManager = dockManager;
			this.dockManager.ClosedPanel += erpDockManagerPanel_ClosedPanel;
			this.dockManager.ClosingPanel += erpDockManagerPanel_ClosingPanel;
		}

		public void OpenPanel(ManagedControl control)
		{
			if (dockManager?.Panels == null)
				throw new ArgumentNullException(nameof(dockManager.Panels));

			var focused = focusPanel(control.Identifier);
			if (focused)
				return;

			DockPanel dockPanel = dockManager.AddPanel(DockingStyle.Float);
			dockPanel.Controls.Add(control);
			control.Dock = DockStyle.Fill;
			control.ControlDockPanel = dockPanel;
			dockPanel.Name = control.Name;
			dockPanel.Text = control.Text;
			dockPanel.DockAsMdiDocument(true);
		}
		public void FocusPanel(Guid identifier)
		{
			focusPanel(identifier);
		}
		private bool focusPanel(Guid identifier)
		{
			foreach (DockPanel panel in dockManager.Panels)
			{
				foreach (var _control in panel.Controls)
				{
					if (_control is ManagedControl && ((ManagedControl)_control).Identifier == identifier)
					{
						dockManager.Form.BeginInvoke(new Action(() =>
						{
							panel.Select();
						}));
						return true;
					}
					else if (_control is ControlContainer _controlContainer)
					{
						foreach (var _innerControl in _controlContainer.Controls)
						{
							if (_innerControl is ManagedControl && ((ManagedControl)_innerControl).Identifier == identifier)
							{
								dockManager.Form.BeginInvoke(new Action(() =>
								{
									panel.Select();
								}));
								return true;
							}
						}
					}
				}
			}

			return false;
		}

		private void erpDockManagerPanel_ClosedPanel(object sender, DockPanelEventArgs e)
		{
			dockManager.RemovePanel(e.Panel);
		}
		private void erpDockManagerPanel_ClosingPanel(object sender, DockPanelCancelEventArgs e)
		{
			try
			{
				if (e.Panel.Controls.Count > 0 &&
					e.Panel.Controls[0] is ControlContainer &&
					e.Panel.Controls[0].Controls.Count > 0 &&
					e.Panel.Controls[0].Controls[0] is ICanClose)
				{
					if (!((ICanClose)e.Panel.Controls[0].Controls[0]).CanClose())
						e.Cancel = true;
				}
			}
			catch { }
		}
	}

	public class ManagedControl : XtraUserControl
	{
		public virtual Guid Identifier { get; }

		private DockPanel controlDockPanel;
		public DockPanel ControlDockPanel
		{
			get { return controlDockPanel; }
			set { controlDockPanel = value; }
		}

		protected void Close()
		{
			if (ControlDockPanel != null)
				ControlDockPanel.Close();
		}
	}

	public interface ICanClose
	{
		bool CanClose();
	}
}
