﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.ModelBinding;
using Sage.PL.WebAPI.Demo.DAL;
using Sage.PL.WebAPI.Demo.Forms;
using Sage.PL.WebAPI.Demo.Model;

namespace Sage.PL.WebAPI.Demo.Services
{
	public class MessagesService
	{
		private List<MessageModel> messages;

		public MessagesService()
		{
			messages = new List<MessageModel>();
		}

		public void AddMessage(string message)
		{
			MessageModel model = new MessageModel();
			model.Message = message;
			model.DateIssue = DateTime.Now;
			model.InnerException = null;
			model.Source = getInvokeMember(2);
			messages.Add(model);

		}
		public void AddMessage(Exception exception)
		{
			MessageModel model = new MessageModel();
			model.Message = exception.Message;
			model.Source = getInvokeMember(2);
			model.StackTrace = exception.StackTrace;
			model.DateIssue = DateTime.Now;
			model.InnerException = exception.InnerException;
			messages.Add(model);
		}
		public void AddMessage<T>(ResponseResult<T> result)
		{
			if (result.Error != null)
			{
				MessageModel model = new MessageModel();
				model.Message = result.Error.Message;
				model.Source = result.RequestUri?.ToString();
				model.StackTrace = result.Error.StackTrace;
				model.DateIssue = DateTime.Now;
				model.InnerException = null;
				model.ModelState = modelStateToString(result.Error.ModelState);
				messages.Add(model);
			}
		}
		public void ClearList()
		{
			messages = new List<MessageModel>();
		}
		public void ShowMessages()
		{
			if (messages.Count == 0)
				return;

			using (FormMessages form = new FormMessages())
			{
				form.LoadData(messages);
				form.ShowDialog();
			}

			messages.Clear();
		}

		private string getInvokeMember(int traceLevel)
		{
			System.Diagnostics.StackFrame frame = new System.Diagnostics.StackFrame(traceLevel);
			var method = frame.GetMethod();
			var type = method.DeclaringType;
			return type != null ? type.FullName : "";
		}
		private string modelStateToString(Dictionary<string, string[]> modelState)
		{

			string message = string.Empty;

			if (modelState == null)
				return message;

			foreach (var key in modelState.Keys)
			{
				foreach (var value in modelState[key])
					message += $"[{key}]: [{value}]{Environment.NewLine}";
			}

			int index = message.LastIndexOf(Environment.NewLine);
			message = message.Substring(0, index);
			return message;
		}
	}
}
